@@ -327,14 +327,14 @@ pub fn replace_all_randn_seeded(circuit: CircuitRc) -> CircuitRc {
         )
         .rc(),
         Circuit::Index(index) => Index::nrc(
-            index.node.clone(),
+            index.node().clone(),
             TensorIndex(
-                zip(&index.index.0, &index.node.info().shape)
+                zip(&index.index.0, index.node().info().shape.clone())
                     .map(|(i, l)| {
                         if let TensorAxisIndex::Tensor(t) = i {
                             return TensorAxisIndex::new_tensor_randint_seeded(
                                 t.shape()[0],
-                                *l,
+                                l,
                                 index.info().device_dtype.clone(),
                                 t.hash_usize().unwrap(),
                             );
