@@ -96,7 +96,7 @@ impl ReplaceMapRc {
                 }
 
                 return iter::once(Self::new(removed_map))
-                    .chain(iter::repeat(self).take(m.nodes.len()).cloned())
+                    .chain(iter::repeat(self).take(m.num_args()).cloned())
                     .collect();
             }
         }
@@ -206,7 +206,7 @@ pub fn expand_node(
         Circuit::SetSymbolicShape(s) => {
             let child_shape = inputs[0].shape();
             let s2: Shape = s
-                .node
+                .node()
                 .shape()
                 .iter()
                 .zip(child_shape.iter().skip(batch_rank))
@@ -274,13 +274,13 @@ pub fn expand_node(
         // I think we just don't do any symbolic shape munging in Index/Scatter?
         Circuit::Index(node) => {
             // for now non-batch non-identity dims can't change
-            for i in 0..node.node.info().rank() {
-                if node.node.info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
+            for i in 0..node.node().info().rank() {
+                if node.node().info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
                     && node.index.0[i] != TensorAxisIndex::IDENT
                 {
                     bail!(ExpandError::FixedIndex {
                         index: node.index.clone(),
-                        old_shape: node.node.info().shape.clone(),
+                        old_shape: node.node().info().shape.clone(),
                         new_shape: inputs[0].info().shape.clone(),
                     });
                 }
@@ -298,13 +298,13 @@ pub fn expand_node(
         }
         Circuit::Scatter(node) => {
             // for now non-batch non-identity dims can't change
-            for i in 0..node.node.info().rank() {
-                if node.node.info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
+            for i in 0..node.node().info().rank() {
+                if node.node().info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
                     && node.index.0[i] != TensorAxisIndex::IDENT
                 {
                     bail!(ExpandError::FixedIndex {
                         index: node.index.clone(),
-                        old_shape: node.node.info().shape.clone(),
+                        old_shape: node.node().info().shape.clone(),
                         new_shape: inputs[0].info().shape.clone(),
                     });
                 }
@@ -332,7 +332,7 @@ pub fn expand_node(
             let inputs = get_expanded_inputs(&vec![true; inputs.len()]).unwrap();
 
             let new_axis = node.axis + br;
-            if !zip(&node.nodes, &inputs)
+            if !zip(node.children(), &inputs)
                 .all(|(old, new)| old.info().shape[node.axis] == new.info().shape[new_axis])
             {
                 bail!(ExpandError::ConcatAxis {
@@ -471,11 +471,11 @@ pub fn expand_node(
                 .chain(node.out_axes.iter().cloned())
                 .collect();
             let new_args: Vec<_> = node
-                .args
+                .in_axes
                 .iter()
                 .zip(inputs)
                 .zip(batch_ranks)
-                .map(|(((_child, ints), inp), r)| {
+                .map(|((ints, inp), r)| {
                     assert!(br >= r);
                     (
                         inp.clone(),
@@ -533,7 +533,7 @@ pub fn expand_node(
         Circuit::Tag(node) => Ok(Tag::nrc(inputs[0].clone(), node.uuid, node.info().name)),
         Circuit::Cumulant(node) => {
             // For now, only supports expansion in easy cases
-            if zip(inputs, &node.nodes)
+            if zip(inputs, node.children())
                 .skip(1)
                 .any(|(new_c, old_c)| new_c.rank() != old_c.rank())
             {
@@ -542,12 +542,12 @@ pub fn expand_node(
             Ok(Cumulant::nrc(inputs.clone(), node.info().name))
         }
         Circuit::Module(node) => {
-            assert_eq!(inputs.len(), node.children().count());
+            assert_eq!(inputs.len(), node.num_children());
             let mut children = inputs.clone();
             let new_nodes = children.split_off(1);
             let spec_circuit = children.pop().unwrap();
 
-            if node.nodes.is_empty() {
+            if node.num_args() == 0 {
                 // handle empty case so we can assume non-empty below
                 return Ok(Module::nrc(
                     vec![],
@@ -598,8 +598,7 @@ pub fn expand_node(
             // [n_0, n_1, n_2, n_3, b_0, b_1, b_2,  w_0, w_1, w_2, w_3]
 
             let current_node_batch_ranks: Vec<_> = node
-                .nodes
-                .iter()
+                .args()
                 .zip(&node.spec.arg_specs)
                 .map(|(node, arg_spec)| node.ndim().checked_sub(arg_spec.symbol.ndim()).unwrap())
                 .collect();
