@@ -1249,8 +1249,6 @@ impl SpecTrait for GeneralFunctionSpec {
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct GeneralFunction {
-    #[pyo3(get)]
-    pub nodes: Vec<CircuitRc>,
     #[pyo3(get)]
     pub spec: GeneralFunctionSpec,
     info: CachedCircuitInfo,
@@ -1286,15 +1284,12 @@ impl GeneralFunction {
             )
         })?;
 
-        let mut out = Self {
-            nodes,
+        let out = Self {
             spec,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, shape, nodes),
             num_non_batchable_output_dims,
             input_batchability,
         };
-        out.info.shape = shape; // set shape so we only compute once
-        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -1306,15 +1301,10 @@ impl GeneralFunction {
 circuit_node_extra_impl!(GeneralFunction, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for GeneralFunction {
-    fn compute_shape(&self) -> Shape {
-        self.info.shape.clone() // set in try new!
-    }
-
     fn device_dtype_override(&self) -> Option<Result<TorchDeviceDtypeOp>> {
         self.spec.get_device_dtype_override(
             &self
-                .nodes
-                .iter()
+                .children()
                 .map(|x| x.info().device_dtype)
                 .collect::<Vec<_>>(),
         )
@@ -1330,28 +1320,13 @@ impl CircuitNodeHashItems for GeneralFunction {
 impl CircuitNode for GeneralFunction {
     circuit_node_auto_impl!("3c655670-b352-4a5f-891c-0d7160609341");
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(self.nodes.iter().cloned())
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Self::try_new(
-            self.nodes
-                .iter()
-                .enumerate()
-                .map(move |(i, circ)| f(i, circ.clone()))
-                .collect::<Result<Vec<_>, _>>()?,
-            self.spec.clone(),
-            self.info().name,
-        )
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Self::try_new(children, self.spec.clone(), self.info().name)
     }
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         let num_batchable_axes = self.info().rank() as u8 - self.num_non_batchable_output_dims;
-        zip(&self.nodes, &self.input_batchability)
+        zip(self.children(), &self.input_batchability)
             .map(|(child, batchable)| {
                 if !batchable {
                     vec![None; child.info().rank()]
@@ -1632,6 +1607,11 @@ impl GeneralFunction {
         .into();
         Self::try_new(vec![x], spec, name)
     }
+
+    #[getter]
+    pub fn nodes(&self) -> Vec<CircuitRc> {
+        self.info().children.clone()
+    }
 }
 
 /// I now think this maybe should have been written in python.
