@@ -113,7 +113,7 @@ macro_rules! circuit_node_self_only_hash_default_impl {
     };
 }
 
-pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
+pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash + Clone {
     // ==== implementable section ===
     //
     // NOTE: ALL FNS IN THIS SECTION *MUST* BE COPIED TO THE CIRCUIT NODE UNION IMPL!
@@ -129,24 +129,15 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     /// if any parent dim is Some, then it must be sound to push down index into only the children w/ that dim
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>>;
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_>;
-
-    fn non_free_children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        self.children()
+    /// free children must come first in children
+    fn num_free_children(&self) -> usize {
+        0
     }
 
-    fn map_children_enumerate<F>(&self, f: F) -> Result<Self>
+    /// assumes children.len() == self.children().len()
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self>
     where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>;
-
-    fn map_non_free_children_enumerate<F>(&self, f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        self.map_children_enumerate(f)
-    }
+        Self: Sized;
 
     fn node_type_uuid(&self) -> [u8; 16];
 
@@ -175,7 +166,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     #[inline]
     fn compute_flags_default(&self) -> CircuitFlags {
         let mut result: CircuitFlags = CircuitFlags::all_true();
-        for child in self.children() {
+        for child in self.children_sl() {
             result &= child.info().flags;
         }
         result
@@ -210,23 +201,61 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         (!self.info().named_axes.is_empty()).then(|| self.info().named_axes.clone())
     }
 
-    fn crc(&self) -> CircuitRc
-    where
-        Self: Clone,
-    {
+    fn crc(&self) -> CircuitRc {
         self.clone().rc()
     }
 
     // TODO: add more map utils for non_free as needed
 
+    fn children(&self) -> std::iter::Cloned<std::slice::Iter<'_, CircuitRc>> {
+        self.info().children.iter().cloned()
+    }
+
+    fn non_free_children(
+        &self,
+    ) -> std::iter::Skip<std::iter::Cloned<std::slice::Iter<'_, CircuitRc>>> {
+        self.children().skip(self.num_free_children())
+    }
+
+    fn children_sl(&self) -> &[CircuitRc] {
+        &self.info().children
+    }
+
     fn num_children(&self) -> usize {
-        // not exactly peak efficiency...
-        self.children().count()
+        self.info().children.len()
+    }
+
+    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
+    where
+        Self: Sized,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
+    {
+        let out: Vec<CircuitRc> = self
+            .children()
+            .enumerate()
+            .map(|(i, x)| f(i, x))
+            .collect::<Result<Vec<_>>>()?;
+        // TODO disabled because reconstructing might need to trigger autoname
+        // if &out == self.children_sl() {
+        //     Ok(self.clone())
+        // } else {
+        self._replace_children(out)
+        // }
+    }
+
+    fn map_non_free_children_enumerate<F>(&self, f: F) -> Result<Self>
+    where
+        Self: Sized,
+        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
+    {
+        let mut f = f;
+        let n = self.num_free_children();
+
+        self.map_children_enumerate(|ix, c| if ix < n { Ok(c) } else { f(ix - n, c) })
     }
 
     fn map_children<F>(&self, mut f: F) -> Result<Self>
     where
-        Self: Sized,
         F: FnMut(CircuitRc) -> Result<CircuitRc>,
     {
         self.map_children_enumerate(|_i, x| f(x))
@@ -301,7 +330,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     }
     fn with_autoname_disabled(&self, autoname_disabled: bool) -> Self
     where
-        Self: Sized + Clone,
+        Self: Sized,
     {
         self.clone()
             .update_info(|info| {
@@ -313,46 +342,28 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
             })
             .unwrap()
     }
-    fn repr(&self) -> Result<String>
-    where
-        Self: Clone,
-    {
+    fn repr(&self) -> Result<String> {
         PrintOptions::repr(&Default::default(), self.crc())
     }
 
-    fn print(&self) -> Result<()>
-    where
-        Self: Clone,
-    {
+    fn print(&self) -> Result<()> {
         rr_util::python_println!("{}", self.repr()?);
         Ok(())
     }
 
-    fn repru(&self) -> String
-    where
-        Self: Clone,
-    {
+    fn repru(&self) -> String {
         self.repr().unwrap()
     }
 
-    fn printu(&self)
-    where
-        Self: Clone,
-    {
+    fn printu(&self) {
         self.print().unwrap()
     }
 
-    fn debug_repr(&self) -> Result<String>
-    where
-        Self: Clone,
-    {
+    fn debug_repr(&self) -> Result<String> {
         print::debug_repr(self.crc())
     }
 
-    fn debug_repru(&self) -> String
-    where
-        Self: Clone,
-    {
+    fn debug_repru(&self) -> String {
         self.debug_repr().unwrap()
     }
 
@@ -372,10 +383,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         self.info().numel_usize()
     }
 
-    fn sum(&self, axes: &[i64], name: Option<Name>) -> Result<Einsum>
-    where
-        Self: Clone,
-    {
+    fn sum(&self, axes: &[i64], name: Option<Name>) -> Result<Einsum> {
         let axes = check_canon_axes_as(self.info().rank(), axes)?;
         Ok(Einsum::new(
             vec![(self.crc(), (0u8..self.info().rank() as u8).collect())],
@@ -386,17 +394,14 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         ))
     }
 
-    fn mean(&self, axes: &[i64], name: Option<Name>, scalar_name: Option<Name>) -> Result<Einsum>
-    where
-        Self: Clone,
-    {
+    fn mean(&self, axes: &[i64], name: Option<Name>, scalar_name: Option<Name>) -> Result<Einsum> {
         let total_size: usize = check_canon_axes_as(self.info().rank(), axes)?
             .into_iter()
             .map(|i| self.info().shape[*i as usize])
             .product();
         let sum = self.sum(axes, None)?;
         Ok(Einsum::new(
-            sum.args
+            sum.args_cloned()
                 .into_iter()
                 .chain(std::iter::once((
                     Scalar::nrc(
@@ -413,10 +418,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         ))
     }
 
-    fn reduce(&self, op_name: Name, axes: &[i64], name: Option<Name>) -> Result<Circuit>
-    where
-        Self: Clone,
-    {
+    fn reduce(&self, op_name: Name, axes: &[i64], name: Option<Name>) -> Result<Circuit> {
         match op_name.str() {
             "mean" => return self.mean(axes, name, None).map(CircuitNode::c),
             "sum" => return self.sum(axes, name).map(CircuitNode::c),
@@ -437,38 +439,23 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         .unwrap()
         .c())
     }
-    fn min_(&self, axes: &[i64], name: Option<Name>) -> Result<Circuit>
-    where
-        Self: Clone,
-    {
+    fn min_(&self, axes: &[i64], name: Option<Name>) -> Result<Circuit> {
         self.reduce("min".into(), axes, name)
     }
-    fn max_(&self, axes: &[i64], name: Option<Name>) -> Result<Circuit>
-    where
-        Self: Clone,
-    {
+    fn max_(&self, axes: &[i64], name: Option<Name>) -> Result<Circuit> {
         self.reduce("max".into(), axes, name)
     }
-    fn add(&self, other: CircuitRc, name: Option<Name>) -> Result<Add>
-    where
-        Self: Clone,
-    {
+    fn add(&self, other: CircuitRc, name: Option<Name>) -> Result<Add> {
         Add::try_new(vec![self.crc(), other], name)
     }
-    fn sub(&self, other: CircuitRc, name: Option<Name>) -> Result<Add>
-    where
-        Self: Clone,
-    {
+    fn sub(&self, other: CircuitRc, name: Option<Name>) -> Result<Add> {
         let other_neg_name = other.info().name.map(|s| format!("neg {}", s).into());
         self.add(
             Einsum::scalar_mul(other, -1.0, other_neg_name, Some("neg_1".into())).rc(),
             name,
         )
     }
-    fn mul(&self, other: CircuitRc, name: Option<Name>) -> Result<Einsum>
-    where
-        Self: Clone,
-    {
+    fn mul(&self, other: CircuitRc, name: Option<Name>) -> Result<Einsum> {
         Einsum::elementwise_broadcasted(vec![self.crc(), other], name)
     }
     fn mul_scalar(
@@ -476,16 +463,10 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         scalar: f64,
         name: Option<Name>,
         scalar_name: Option<Name>,
-    ) -> Result<Einsum>
-    where
-        Self: Clone,
-    {
+    ) -> Result<Einsum> {
         self.mul(Scalar::nrc(scalar, sv![], scalar_name), name)
     }
-    fn index(&self, index: TensorIndex, name: Option<Name>) -> Result<Index>
-    where
-        Self: Clone,
-    {
+    fn index(&self, index: TensorIndex, name: Option<Name>) -> Result<Index> {
         Index::try_new(self.crc(), index, name)
     }
     fn expand_at_axes(
@@ -493,30 +474,21 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         axes: Vec<usize>,
         counts: Vec<usize>,
         name: Option<Name>,
-    ) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    ) -> Result<Rearrange> {
         Ok(Rearrange::new(
             self.crc(),
             RearrangeSpec::expand_at_axes(self.info().rank(), axes, counts)?,
             name,
         ))
     }
-    fn unsqueeze(&self, axes: Vec<usize>, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn unsqueeze(&self, axes: Vec<usize>, name: Option<Name>) -> Result<Rearrange> {
         Ok(Rearrange::new(
             self.crc(),
             RearrangeSpec::unsqueeze(self.info().rank(), axes)?,
             name,
         ))
     }
-    fn squeeze(&self, axes: PyOpAtAxes, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn squeeze(&self, axes: PyOpAtAxes, name: Option<Name>) -> Result<Rearrange> {
         let axes = check_canon_axes_as(
             self.info().rank(),
             &reduction_to_ints(Some(axes), self.info().rank()),
@@ -549,26 +521,17 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
             name,
         )
     }
-    fn flatten(&self, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn flatten(&self, name: Option<Name>) -> Result<Rearrange> {
         Ok(Rearrange::new(
             self.crc(),
             RearrangeSpec::flatten_usize(self.ndim())?,
             name,
         ))
     }
-    fn unflatten(&self, shape: Shape, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn unflatten(&self, shape: Shape, name: Option<Name>) -> Result<Rearrange> {
         Rearrange::unflatten(self.crc(), shape, name)
     }
-    fn unflatten_axis(&self, axis: i64, shape: Shape, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn unflatten_axis(&self, axis: i64, shape: Shape, name: Option<Name>) -> Result<Rearrange> {
         Ok(Rearrange::new(
             self.crc(),
             RearrangeSpec::unflatten_axis(self.ndim(), axis, shape)?,
@@ -580,26 +543,17 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         axis: usize,
         shape: Shape,
         name: Option<Name>,
-    ) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    ) -> Result<Rearrange> {
         Ok(Rearrange::new(
             self.crc(),
             RearrangeSpec::unflatten_axis_usize(self.ndim(), axis, shape)?,
             name,
         ))
     }
-    fn rearrange(&self, spec: RearrangeSpec, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn rearrange(&self, spec: RearrangeSpec, name: Option<Name>) -> Result<Rearrange> {
         Rearrange::try_new(self.crc(), spec, name)
     }
-    fn rearrange_str(&self, string: &str, name: Option<Name>) -> Result<Rearrange>
-    where
-        Self: Clone,
-    {
+    fn rearrange_str(&self, string: &str, name: Option<Name>) -> Result<Rearrange> {
         Rearrange::from_string(self.crc(), string, name)
     }
     fn add_suffix(self, suffix: Option<&str>) -> Self
@@ -636,20 +590,6 @@ pub trait CircuitNodeAutoName: CircuitNode {
     }
 }
 
-pub trait CircuitNodeDefer: CircuitNodeInit {
-    fn as_trait_obj(&self) -> &dyn CircuitNode;
-    fn map_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>;
-    fn map_non_free_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>;
-    fn custom_c(self) -> Circuit;
-    fn custom_rc(self) -> CircuitRc;
-}
-
 pub trait CircuitNodeUnion {
     type TypeTag;
     fn variant_string(&self) -> String;
@@ -869,6 +809,7 @@ macro_rules! define_circuit_union_impl {
         }
 
 
+
         paste::paste! {
         impl $crate::CircuitNodeInit for $name {
             fn init_info_impl(self, is_initial: bool) -> Result<Self> {
@@ -890,35 +831,38 @@ macro_rules! define_circuit_union_impl {
             }
         }
 
-        impl $crate::CircuitNodeDefer for $name {
-            #[inline] // hopefully inlined away?
-            fn as_trait_obj(&self) -> &dyn $crate::CircuitNode {
-                [<$name:snake _map>]!(self; node => node)
+        impl $crate::CircuitNodeSelfOnlyHash for $name {
+            fn compute_self_only_hash(&self, hasher: &mut blake3::Hasher) {
+                [<$name:snake _map>]!(self; node => node.compute_self_only_hash(hasher))
+            }
+        }
+
+        impl $crate::CircuitNode for $name {
+            fn info(&self) -> &CachedCircuitInfo {
+                [<$name:snake _map>]!(self; node => node.info())
             }
 
-            fn map_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
-            where
-                F: FnMut(usize,$crate::CircuitRc) -> Result<$crate::CircuitRc>,
-            {
-                [<$name:snake _map_construct>]!(self;
-                    (node, construct) => $crate::CircuitNode::map_children_enumerate(node, f).map(|v| construct(v)))
+            fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+                [<$name:snake _map_construct>]!(self; (node, construct) => Ok(construct(node._replace_children(children)?)))
             }
 
-            fn map_non_free_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
-            where
-                F: FnMut(usize,$crate::CircuitRc) -> Result<$crate::CircuitRc>,
-            {
-                [<$name:snake _map_construct>]!(self;
-                    (node, construct) => $crate::CircuitNode::map_non_free_children_enumerate(node, f).map(|v| construct(v)))
+            fn node_type_uuid(&self) -> [u8; 16] {
+                [<$name:snake _map>]!(self; node => node.node_type_uuid())
             }
 
-            fn custom_c(self) -> $crate::Circuit {
-                [<$name:snake _map>]!(self; node => node.c())
+            fn self_flops(&self) -> BigUint {
+                [<$name:snake _map>]!(self; node => node.self_flops())
             }
 
-            fn custom_rc(self) -> $crate::CircuitRc {
-                [<$name:snake _map>]!(self; node => node.rc())
+            fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+                [<$name:snake _map>]!(self; node => node.eval_tensors(tensors))
             }
+
+            // TODO: can return self for Circuit.c()
+            fn c(self) -> Circuit { to_circuit_maybe_short_circuit!($name, self, [<$name:snake _map>]!(self; node => node.c())) }
+            fn rc(self) -> CircuitRc { [<$name:snake _map>]!(self; node => node.rc()) }
+            fn get_autoname(&self) -> Result<Option<Name>> { [<$name:snake _map>]!(self; node => node.get_autoname()) }
+            fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> { [<$name:snake _map>]!(self; node => node.child_axis_map()) }
         }
         }
 
@@ -955,6 +899,14 @@ macro_rules! define_circuit_union_impl {
         )*
     }
 }
+macro_rules! to_circuit_maybe_short_circuit {
+    (Circuit, $self:ident, $e:expr) => {
+        $self
+    };
+    ($n:ident, $self:ident, $e:expr) => {
+        $e
+    };
+}
 macro_rules! define_circuit {
     [$($x:ident),+ $(,)?] => {
         define_circuit_union_impl!(Circuit {$($x,)*});
@@ -1179,70 +1131,7 @@ macro_rules! circuit_node_auto_impl {
     };
 }
 
-impl<T: CircuitNodeDefer> CircuitNodeSelfOnlyHash for T {
-    fn compute_self_only_hash(&self, hasher: &mut blake3::Hasher) {
-        self.as_trait_obj().compute_self_only_hash(hasher)
-    }
-}
-
-// UPDATE ME WHEN YOU CHANGE CircuitNode Trait!!!
-impl<T: CircuitNodeDefer> CircuitNode for T {
-    fn info(&self) -> &CachedCircuitInfo {
-        self.as_trait_obj().info()
-    }
-
-    fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        self.as_trait_obj().child_axis_map()
-    }
-
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        self.as_trait_obj().children()
-    }
-
-    fn non_free_children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        self.as_trait_obj().non_free_children()
-    }
-
-    fn map_children_enumerate<F>(&self, f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        self.map_children_enumerate_impl(f)
-    }
-
-    fn map_non_free_children_enumerate<F>(&self, f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        self.map_non_free_children_enumerate_impl(f)
-    }
-
-    fn node_type_uuid(&self) -> [u8; 16] {
-        self.as_trait_obj().node_type_uuid()
-    }
-
-    fn self_flops(&self) -> BigUint {
-        self.as_trait_obj().self_flops()
-    }
-
-    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
-        self.as_trait_obj().eval_tensors(tensors)
-    }
-
-    fn c(self) -> Circuit {
-        self.custom_c()
-    }
-
-    fn rc(self) -> CircuitRc {
-        self.custom_rc()
-    }
-
-    fn get_autoname(&self) -> Result<Option<Name>> {
-        self.as_trait_obj().get_autoname()
-    }
-}
-
-#[derive(Clone, Default)]
+#[derive(Clone)]
 pub struct CachedCircuitInfo {
     pub shape: Shape,
     pub flags: CircuitFlags,
@@ -1252,6 +1141,28 @@ pub struct CachedCircuitInfo {
     pub symbolic_size_constraints: SymbolicSizeConstraints,
     pub free_symbols: Option<Arc<IndexSet<Symbol>>>,
     pub name: Option<Name>,
+    pub children: Vec<CircuitRc>,
+}
+
+impl CachedCircuitInfo {
+    // leaves some fields undefined/default for now... TODO move more stuff here
+    pub fn incomplete(
+        name: Option<Name>,
+        shape: Shape,
+        children: Vec<CircuitRc>,
+    ) -> CachedCircuitInfo {
+        CachedCircuitInfo {
+            shape,
+            flags: Default::default(),
+            hash: Default::default(),
+            device_dtype: Default::default(),
+            named_axes: Default::default(),
+            symbolic_size_constraints: Default::default(),
+            free_symbols: Default::default(),
+            name,
+            children,
+        }
+    }
 }
 
 #[derive(Clone, Copy, Hash, Debug, Eq, PartialEq)]
@@ -1600,6 +1511,50 @@ pub enum TensorEvalError {
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub struct CircuitRc(Arc<Circuit>);
 
+impl CircuitNodeSelfOnlyHash for CircuitRc {
+    fn compute_self_only_hash(&self, hasher: &mut blake3::Hasher) {
+        self.0.compute_self_only_hash(hasher)
+    }
+}
+impl CircuitNode for CircuitRc {
+    fn info(&self) -> &CachedCircuitInfo {
+        self.0.info()
+    }
+
+    fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
+        self.0.child_axis_map()
+    }
+
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        self.0._replace_children(children).map(|x| x.rc())
+    }
+
+    fn node_type_uuid(&self) -> [u8; 16] {
+        self.0.node_type_uuid()
+    }
+
+    fn self_flops(&self) -> BigUint {
+        self.0.self_flops()
+    }
+
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        self.0.eval_tensors(tensors)
+    }
+
+    fn c(self) -> Circuit {
+        arc_unwrap_or_clone(self.0)
+    }
+    fn rc(self) -> CircuitRc {
+        self
+    }
+
+    // fn static_node_type_uuid() -> [u8; 16] { self.0.static_node_type_uuid() }
+
+    fn get_autoname(&self) -> Result<Option<Name>> {
+        self.0.get_autoname()
+    }
+}
+
 impl fmt::Debug for CircuitRc {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt::Debug::fmt(&***self, f)
@@ -1756,7 +1711,7 @@ where
             f(circ.clone())?;
 
             let children = if recur_into_free {
-                circ.children()
+                circ.children().skip(0)
             } else {
                 circ.non_free_children()
             };
@@ -1974,6 +1929,14 @@ impl IntoPy<PyObject> for CircuitRc {
     }
 }
 
+impl IntoPy<PyObject> for &CircuitRc {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        {
+            (*self.0).clone().into_py(py)
+        }
+    }
+}
+
 impl<'source> FromPyObject<'source> for CircuitRc {
     fn extract(circuit_obj: &'source PyAny) -> PyResult<Self> {
         {
@@ -2032,39 +1995,6 @@ impl CircuitNodeInit for CircuitRc {
     }
 }
 
-impl CircuitNodeDefer for CircuitRc {
-    fn as_trait_obj(&self) -> &dyn CircuitNode {
-        // deref to avoid infinite recursion
-        (**self).as_trait_obj()
-    }
-
-    fn map_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        (**self).map_children_enumerate(f).map(CircuitNode::rc)
-    }
-    fn map_non_free_children_enumerate_impl<F>(&self, f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        (**self)
-            .map_non_free_children_enumerate(f)
-            .map(CircuitNode::rc)
-    }
-
-    fn custom_c(self) -> Circuit {
-        arc_unwrap_or_clone(self.0)
-    }
-
-    // fast custom impl
-    fn custom_rc(self) -> CircuitRc {
-        self
-    }
-}
-
 #[pyclass(subclass, name = "Circuit")]
 #[derive(Clone, Debug)]
 pub struct PyCircuitBase(CircuitRc);
@@ -2144,7 +2074,7 @@ impl PyCircuitBase {
 
     #[getter]
     fn children(&self) -> Vec<CircuitRc> {
-        self.0.children().collect()
+        self.0.info().children.clone()
     }
 
     #[getter]
@@ -2361,7 +2291,7 @@ impl PyCircuitBase {
         self.expand_at_axes(axes, None, name)
     }
     fn squeeze(&self, axes: PyOpAtAxes, name: Option<Name>) -> Result<Rearrange> {
-        CircuitNode::squeeze(&self.clone().crc(), axes, name)
+        self.0.squeeze(axes, name)
     }
     fn flatten(&self, name: Option<Name>) -> Result<Rearrange> {
         self.0.flatten(name)
@@ -2492,7 +2422,15 @@ pub mod prelude {
 }
 
 pub fn circuit_rust_to_py(circ: CircuitRc) -> PyObject {
-    Python::with_gil(|py| PY_CIRCUIT_ITEMS.rust_to_py.call(py, (circ,), None).unwrap())
+    Python::with_gil(|py| {
+        PY_CIRCUIT_ITEMS
+            .rust_to_py
+            .call(py, (circ,), None)
+            .unwrap_or_else(|e| {
+                e.print(py);
+                panic!()
+            })
+    })
 }
 
 /// casts tensors whith None device/dtype to specified device/dtype
