@@ -75,7 +75,7 @@ pub fn propagate_named_axes(
             let child_axis_names: Vec<NamedAxes> = if abort_on_branch
                 && circuit
                     .as_einsum()
-                    .map(|x| x.args.iter().any(|(_, i)| !is_unique(i)))
+                    .map(|x| x.args().any(|(_, i)| !is_unique(&i)))
                     .unwrap_or(false)
             {
                 circuit
@@ -164,8 +164,7 @@ pub fn get_axis_leaves(
             if let Circuit::Module(m) = &**x {
                 if axis_of_name(&x, name).unwrap() < m.aligned_batch_shape().len() {
                     let nodes2 = m
-                        .nodes
-                        .iter()
+                        .args()
                         .map(|c| f(ra, rna, seen, name, c.clone()))
                         .collect::<Result<Vec<CircuitRc>>>()?;
                     return Ok(Module::nrc(nodes2, m.spec.clone(), m.info().name));
