@@ -700,7 +700,7 @@ impl PrintOptions {
                 format!("{:.}", scalar.value)
             }
             Circuit::Rearrange(rearrange) => rearrange.spec.to_maybe_fancy_einops_string(
-                rearrange.node.info().named_axes.clone(),
+                rearrange.node().info().named_axes.clone(),
                 rearrange.info().named_axes.clone(),
             ),
             Circuit::Einsum(einsum) => einsum.get_spec().to_maybe_fancy_string(
@@ -763,8 +763,8 @@ impl PrintOptions {
             Circuit::StoredCumulantVar(scv) => {
                 format!(
                     "{}|{}",
-                    scv.cumulants
-                        .keys()
+                    scv.cumulant_ixs
+                        .iter()
                         .map(|k| k.to_string())
                         .collect::<Vec<_>>()
                         .join(", "),
@@ -1276,8 +1276,7 @@ pub fn last_child_arrows(last_child: &Vec<bool>, is_output: bool, arrows: bool)
 
 pub fn get_child_comments(parent: &Circuit, child_pos: usize) -> Vec<String> {
     vec![match parent {
-        Circuit::Einsum(ein) => ein.args[child_pos]
-            .1
+        Circuit::Einsum(ein) => ein.in_axes[child_pos]
             .iter()
             .map(|int| ALPHABET[*int as usize].clone())
             .join(""),
@@ -1296,7 +1295,7 @@ pub fn get_child_comments(parent: &Circuit, child_pos: usize) -> Vec<String> {
             }
         },
         Circuit::StoredCumulantVar(scv) => {
-            format!("K{}", scv.cumulants.keys().nth(child_pos).unwrap())
+            format!("K{}", scv.cumulant_ixs.iter().nth(child_pos).unwrap())
         }
         Circuit::Module(_) => {
             if child_pos == 0 {
