@@ -243,18 +243,18 @@ impl PrintHtmlOptions {
 #[pymethods]
 impl PrintHtmlOptions {
     #[new]
-    #[args(
-        shape_only_when_necessary = "PrintHtmlOptions::default().shape_only_when_necessary",
-        traversal = "PrintHtmlOptions::default().traversal.unwrap()",
-        display_copy_button = "PrintHtmlOptions::default().display_copy_button",
-        colorer = "PrintHtmlOptions::default().colorer",
-        primary_color = "PrintHtmlOptions::default().primary_color",
-        number_leaves = "PrintHtmlOptions::default().number_leaves",
-        keep_all_cells = "PrintHtmlOptions::default().keep_all_cells",
-        overall_traversal = "PrintHtmlOptions::default().overall_traversal.unwrap()",
-        comment_arg_names = "PrintOptions::default().comment_arg_names",
-        commenters = "PrintOptions::default().commenters"
-    )] // None are mapped to the default
+    #[pyo3(signature=(
+        shape_only_when_necessary = PrintHtmlOptions::default().shape_only_when_necessary,
+        traversal = PrintHtmlOptions::default().traversal.unwrap(),
+        display_copy_button = PrintHtmlOptions::default().display_copy_button,
+        colorer = PrintHtmlOptions::default().colorer,
+        primary_color = PrintHtmlOptions::default().primary_color,
+        number_leaves = PrintHtmlOptions::default().number_leaves,
+        keep_all_cells = PrintHtmlOptions::default().keep_all_cells,
+        overall_traversal = PrintHtmlOptions::default().overall_traversal.unwrap(),
+        comment_arg_names = PrintOptions::default().comment_arg_names,
+        commenters = PrintOptions::default().commenters
+    ))] // None are mapped to the default
     pub fn py_new(
         shape_only_when_necessary: bool,
         traversal: Option<OpaqueIterativeMatcherVal>,
@@ -282,18 +282,18 @@ impl PrintHtmlOptions {
         Ok(result)
     }
 
-    #[args(
-        shape_only_when_necessary = "Default::default()",
-        traversal = "Default::default()",
-        colorer = "MaybeNotSet(None)",
-        primary_color = "Default::default()",
-        display_copy_button = "Default::default()",
-        number_leaves = "Default::default()",
-        keep_all_cells = "Default::default()",
-        overall_traversal = "Default::default()",
-        comment_arg_names = "Default::default()",
-        commenters = "Default::default()"
-    )]
+    #[pyo3(signature=(
+        shape_only_when_necessary = Default::default(),
+        traversal = Default::default(),
+        display_copy_button = Default::default(),
+        colorer = MaybeNotSet(None),
+        primary_color = Default::default(),
+        number_leaves = Default::default(),
+        keep_all_cells = Default::default(),
+        overall_traversal = Default::default(),
+        comment_arg_names = Default::default(),
+        commenters = Default::default()
+    ))]
     pub fn evolve(
         &self,
         shape_only_when_necessary: MaybeNotSet<bool>,
@@ -347,7 +347,7 @@ impl PrintHtmlOptions {
         self.overall_traversal = overall_traversal
     }
 
-    #[args(circuits = "*")]
+    #[pyo3(signature=(*circuits))]
     pub fn print(&self, circuits: Vec<CircuitRc>) -> Result<()> {
         let html = self.repr(circuits)?;
         Python::with_gil(|py| -> Result<()> {
@@ -368,7 +368,7 @@ impl PrintHtmlOptions {
         })
     }
 
-    #[args(circuits = "*")]
+    #[pyo3(signature=(*circuits))]
     pub fn repr(&self, circuits: Vec<CircuitRc>) -> Result<String> {
         let mut html = HTML_HEADER.to_owned();
         let mut primary_color = self.primary_color.clone();
@@ -464,7 +464,7 @@ impl PrintHtmlOptions {
     }
 
     #[staticmethod]
-    #[args(circuits = "*", depth = "3")]
+    #[pyo3(signature=(*circuits, depth = 3))]
     pub fn print_depth(circuits: Vec<CircuitRc>, depth: usize) -> Result<()> {
         PrintHtmlOptions {
             traversal: Some(OpaqueIterativeMatcherVal::for_end_depth(depth)),
