@@ -20,20 +20,15 @@ use crate::{
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct SetSymbolicShape {
-    #[pyo3(get)]
-    pub node: CircuitRc,
     info: CachedCircuitInfo,
 }
 
 impl SetSymbolicShape {
     #[apply(new_rc_unwrap)]
     pub fn try_new(node: CircuitRc, shape: Shape, name: Option<Name>) -> Result<Self> {
-        let mut out = Self {
-            node,
-            info: Default::default(),
+        let out = Self {
+            info: CachedCircuitInfo::incomplete(name, shape, vec![node]),
         };
-        out.info.shape = shape;
-        out.info.name = name;
         out.initial_init_info()
     }
 }
@@ -41,13 +36,9 @@ impl SetSymbolicShape {
 circuit_node_extra_impl!(SetSymbolicShape, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for SetSymbolicShape {
-    fn compute_shape(&self) -> Shape {
-        self.info().shape.clone()
-    }
-
     fn symbolic_size_constraints_extra(&self) -> Result<Vec<SymbolicSizeConstraint>> {
         let out = self
-            .node
+            .node()
             .shape()
             .iter()
             .zip(self.shape())
@@ -60,7 +51,7 @@ impl CircuitNodeComputeInfoImpl for SetSymbolicShape {
     }
 
     fn compute_flags(&self) -> CircuitFlags {
-        self.node.compute_flags_default() & !CircuitFlags::IS_EXPLICITLY_COMPUTABLE
+        self.node().compute_flags_default() & !CircuitFlags::IS_EXPLICITLY_COMPUTABLE
     }
 }
 
@@ -75,20 +66,13 @@ impl CircuitNodeHashItems for SetSymbolicShape {
 impl CircuitNode for SetSymbolicShape {
     circuit_node_auto_impl!("25332a58-39d6-443d-b225-65c810a09aa4");
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(std::iter::once(self.node.clone()))
-    }
-
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         vec![(0..self.rank()).map(Some).collect()]
     }
 
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
         Self::try_new(
-            f(0, self.node.clone())?,
+            children[0].clone(),
             self.info().shape.clone(),
             self.info().name,
         )
@@ -106,7 +90,7 @@ impl CircuitNodeAutoName for SetSymbolicShape {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
     fn auto_name(&self) -> Option<Name> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node().clone())
             .map(|n| (n.string() + " set_shape").into())
     }
 }
@@ -160,4 +144,9 @@ impl SetSymbolicShape {
             .collect();
         Self::some_set_neq(node, shape, name)
     }
+
+    #[getter]
+    pub fn node(&self) -> CircuitRc {
+        self.info.children[0].clone()
+    }
 }
