@@ -43,12 +43,11 @@ use crate::circuit_optimizer::OptimizationContext;
 pub fn add_flatten_once(add: &Add) -> Option<Add> {
     let mut did_anything = false;
     let new_operands: Vec<CircuitRc> = add
-        .nodes
-        .iter()
-        .flat_map(|node| match &***node {
+        .children()
+        .flat_map(|node| match &**node {
             Circuit::Add(inner_add) => {
                 did_anything = true;
-                inner_add.nodes.clone()
+                inner_add.children_sl().to_vec()
             }
             _ => {
                 vec![node.clone()]
@@ -63,9 +62,12 @@ pub fn add_flatten_once(add: &Add) -> Option<Add> {
 
 #[pyfunction]
 pub fn add_collapse_scalar_inputs(add: &Add) -> Option<Add> {
-    let nodes = add.nodes.iter().map(|x| (**x).clone());
-    let (scalar_inputs, non_scalar_inputs): (Vec<Scalar>, Vec<Arc<Circuit>>) =
-        filter_by_variant!(nodes, Circuit, Scalar, Scalar);
+    let (scalar_inputs, non_scalar_inputs): (Vec<Scalar>, Vec<Arc<Circuit>>) = filter_by_variant!(
+        add.children().map(|x| ((*x).clone())),
+        Circuit,
+        Scalar,
+        Scalar
+    );
     if scalar_inputs.len() <= 1 {
         None
     } else {
@@ -82,18 +84,17 @@ pub fn add_collapse_scalar_inputs(add: &Add) -> Option<Add> {
 pub fn add_elim_zeros(add: &Add) -> Option<Add> {
     let mut did_anything = false;
     let new_operands = add
-        .nodes
-        .iter()
-        .filter_map(|node| match &***node {
+        .children()
+        .filter_map(|node| match &**node {
             Circuit::Scalar(scalar) => {
                 if scalar.value == 0.0 && scalar.info().rank() == 0 {
                     did_anything = true;
                     None
                 } else {
-                    Some(node.clone())
+                    Some(node)
                 }
             }
-            _ => Some(node.clone()),
+            _ => Some(node),
         })
         .collect();
     if !did_anything {
@@ -107,11 +108,11 @@ pub fn add_elim_zeros(add: &Add) -> Option<Add> {
 pub fn add_deduplicate(add: &Add) -> Option<Add> {
     let mut duplicate_counts: HashMap<HashBytes, i32> = HashMap::default();
     let mut hash_to_node: HashMap<HashBytes, CircuitRc> = HashMap::default();
-    for node in add.nodes.iter() {
+    for node in add.children() {
         *duplicate_counts.entry(node.info().hash).or_insert(0) += 1;
         hash_to_node.insert(node.info().hash, node.clone());
     }
-    if duplicate_counts.len() == add.nodes.len() {
+    if duplicate_counts.len() == add.num_children() {
         None
     } else {
         let deduped_inputs = hash_to_node
@@ -138,12 +139,10 @@ pub fn add_deduplicate(add: &Add) -> Option<Add> {
 
 #[pyfunction]
 pub fn remove_add_few_input(add: &Add) -> Option<CircuitRc> {
-    if add.nodes.is_empty() {
-        Some(Scalar::new(0f64, add.info().shape.clone(), add.info().name).rc())
-    } else if add.nodes.len() == 1 {
-        Some(add.nodes[0].clone())
-    } else {
-        None
+    match add.num_children() {
+        0 => Some(Scalar::new(0f64, add.info().shape.clone(), add.info().name).rc()),
+        1 => Some(add.children_sl()[0].clone()),
+        _ => None,
     }
 }
 
@@ -170,9 +169,8 @@ pub fn make_einsum_ints_same_one_layer_and_int_info(
     // don't know the number of result ints, so UF for really high amount?
     let mut unionfind = UnionFind::new(300);
     let int_maps: Vec<Option<HashMap<u8, u8>>> = einsum
-        .args
-        .iter()
-        .map(|(node, ints)| match &***node {
+        .args()
+        .map(|(node, ints)| match &**node {
             Circuit::Einsum(einsum) => {
                 let mut bottom_int_map: HashMap<u8, u8> = HashMap::default();
 
@@ -199,21 +197,20 @@ pub fn make_einsum_ints_same_one_layer_and_int_info(
         })
         .collect();
 
-    let new_args = zip(&einsum.args, int_maps.clone())
+    let new_args = zip(einsum.args(), int_maps.clone())
         .map(|((node, ints), int_map_op)| {
             let new_operand_axes = ints
                 .iter()
                 .map(|x| unionfind.find(top_int_map[x] as usize) as u8)
                 .collect();
-            match &***node {
+            match &**node {
                 Circuit::Einsum(inner) => {
                     let int_map = int_map_op.unwrap();
                     let new_args_inner = inner
-                        .args
-                        .iter()
+                        .args()
                         .map(|(child, inner_op_ints)| {
                             (
-                                child.clone(),
+                                child,
                                 inner_op_ints
                                     .iter()
                                     .map(|i| unionfind.find(int_map[i] as usize) as u8)
@@ -258,20 +255,18 @@ pub fn make_einsum_ints_same_one_layer_and_int_info(
 #[pyfunction]
 pub fn einsum_flatten_once(einsum: &Einsum) -> Option<Einsum> {
     if einsum
-        .args
-        .iter()
-        .all(|(node, _ints)| !matches!(&***node, Circuit::Einsum(_)))
+        .children()
+        .all(|node| !matches!(**node, Circuit::Einsum(_)))
     {
         return None;
     }
     let with_ints_same = make_einsum_ints_same_one_layer(einsum);
     Some(Einsum::new(
         with_ints_same
-            .args
-            .into_iter()
+            .args()
             .flat_map(|(node, ints)| match &**node {
-                Circuit::Einsum(einsum) => einsum.args.clone(),
-                _ => vec![(node, ints)],
+                Circuit::Einsum(einsum) => einsum.args_cloned(),
+                _ => vec![(node, ints.clone())],
             })
             .collect(),
         with_ints_same.out_axes,
@@ -282,7 +277,7 @@ pub fn einsum_flatten_once(einsum: &Einsum) -> Option<Einsum> {
 /// nests einsum (similar to "rearrange_muls") according to an einops-style path with recycled IDs
 #[pyfunction]
 pub fn einsum_nest_path(einsum: &Einsum, path: Vec<Vec<usize>>) -> Einsum {
-    let mut stack = einsum.args.clone();
+    let mut stack = einsum.args_cloned();
     for contraction in path {
         let mut contraction = contraction.clone();
         contraction.sort();
@@ -340,8 +335,8 @@ pub fn einsum_nest_optimize(einsum: &Einsum, context: &mut OptimizationContext)
         None,
         Some(context.settings.max_single_tensor_memory / einsum.info().device_dtype.size()),
         None,
-        Some(filter_to_idx(&einsum.args, &|(x, _)| {
-            matches!(&***x, Circuit::Array(_))
+        Some(filter_to_idx(einsum.children(), |x| {
+            matches!(***x, Circuit::Array(_))
         })),
     )?;
     // let path = einsum.get_spec().optimize_dp(None,None,None);
@@ -467,12 +462,12 @@ pub fn make_broadcast(node: &CircuitRc, out_shape: &Shape) -> Result<CircuitRc>
 
 #[pyfunction]
 pub fn einsum_elim_identity(einsum: &Einsum) -> Option<CircuitRc> {
-    if einsum.args.len() == 1
-        && einsum.out_axes[..] == einsum.args[0].1[..]
+    if einsum.num_children() == 1
+        && einsum.out_axes[..] == einsum.in_axes[0][..]
         && is_unique(&einsum.out_axes)
     {
-        Some(einsum.args[0].0.clone())
-    } else if einsum.args.is_empty() {
+        Some(einsum.children_sl()[0].clone())
+    } else if einsum.num_children() == 0 {
         Some(Scalar::new(1.0, sv![], einsum.info().name).rc())
     } else {
         None
@@ -482,15 +477,15 @@ pub fn einsum_elim_identity(einsum: &Einsum) -> Option<CircuitRc> {
 #[pyfunction]
 pub fn index_merge_scalar(index: &Index) -> Option<CircuitRc> {
     index
-        .node
+        .node()
         .as_scalar()
         .map(|scalar| Scalar::new(scalar.value, index.info().shape.clone(), index.info().name).rc())
 }
 
 #[pyfunction]
 pub fn index_elim_identity(index: &Index) -> Option<CircuitRc> {
-    if index.index.is_identity(&index.node.info().shape) {
-        Some(index.node.clone())
+    if index.index.is_identity(&index.node().info().shape) {
+        Some(index.node().clone())
     } else {
         None
     }
@@ -498,9 +493,9 @@ pub fn index_elim_identity(index: &Index) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn index_fuse(index: &Index) -> Option<Index> {
-    index.node.as_index().and_then(|inner| {
+    index.node().as_index().and_then(|inner| {
         Some(Index::new(
-            inner.node.clone(),
+            inner.node().clone(),
             compose(&index.index, &inner.index)?,
             index.info().name.map(|n| {
                 if n.ends_with(" idx idx") {
@@ -515,7 +510,7 @@ pub fn index_fuse(index: &Index) -> Option<Index> {
 
 #[pyfunction]
 pub fn rearrange_merge_scalar(rearrange: &Rearrange) -> Option<CircuitRc> {
-    rearrange.node.as_scalar().map(|scalar| {
+    rearrange.node().as_scalar().map(|scalar| {
         Scalar::new(
             scalar.value,
             rearrange.info().shape.clone(),
@@ -528,7 +523,7 @@ pub fn rearrange_merge_scalar(rearrange: &Rearrange) -> Option<CircuitRc> {
 #[pyfunction]
 pub fn rearrange_elim_identity(rearrange: &Rearrange) -> Option<CircuitRc> {
     if rearrange.spec.is_identity() {
-        Some(rearrange.node.clone())
+        Some(rearrange.node().clone())
     } else {
         None
     }
@@ -537,9 +532,8 @@ pub fn rearrange_elim_identity(rearrange: &Rearrange) -> Option<CircuitRc> {
 #[pyfunction]
 pub fn concat_merge_uniform(concat: &Concat) -> Option<CircuitRc> {
     let maybe_scalars: Vec<Option<f64>> = concat
-        .nodes
-        .iter()
-        .map(|x| match &***x {
+        .children()
+        .map(|x| match &**x {
             Circuit::Scalar(scalar) => Some(scalar.value),
             _ => None,
         })
@@ -560,8 +554,8 @@ pub fn concat_merge_uniform(concat: &Concat) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn concat_elim_identity(concat: &Concat) -> Option<CircuitRc> {
-    if concat.nodes.len() == 1 {
-        Some(concat.nodes[0].clone())
+    if concat.num_children() == 1 {
+        Some(concat.children_sl()[0].clone())
     } else {
         None
     }
@@ -619,7 +613,7 @@ pub fn remove_axes_gen(
                 (*rearrange.spec).clone()
             };
             Some(Rearrange::nrc_elim_identity(
-                rearrange.node.clone(),
+                rearrange.node().clone(),
                 spec.filter_out_axes(axes).unwrap(),
                 get_name(rearrange),
             ))
@@ -653,10 +647,10 @@ pub fn remove_axes_gen(
 
 #[pyfunction]
 pub fn generalfunction_pull_removable_axes(node: &GeneralFunction) -> Option<CircuitRc> {
-    if node.nodes.len() != 1 {
+    if node.num_children() != 1 {
         return None;
     }
-    let removable_axes_inp = get_removable_axes(&node.nodes[0]);
+    let removable_axes_inp = get_removable_axes(&node.children_sl()[0]);
     let removable_batchable_inp: HashSet<usize> = removable_axes_inp
         .iter()
         .filter(|x| (**x + node.num_non_batchable_output_dims as usize) < node.info().rank())
@@ -666,7 +660,7 @@ pub fn generalfunction_pull_removable_axes(node: &GeneralFunction) -> Option<Cir
         return None;
     }
     let new_generalfunction = GeneralFunction::new(
-        vec![remove_axes(&node.nodes[0], &removable_batchable_inp).unwrap()],
+        vec![remove_axes(&node.children_sl()[0], &removable_batchable_inp).unwrap()],
         node.spec.clone(),
         node.info().name,
     );
@@ -680,7 +674,7 @@ pub fn generalfunction_pull_removable_axes(node: &GeneralFunction) -> Option<Cir
 #[pyfunction]
 pub fn concat_pull_removable_axes(node: &Concat) -> Option<CircuitRc> {
     let removable_axes_per: Vec<HashSet<usize>> =
-        node.nodes.iter().map(get_removable_axes).collect();
+        node.children().map(|x| get_removable_axes(&x)).collect();
     let removable_axes = intersection_all(&removable_axes_per);
     let removable_non_axis: HashSet<usize> = removable_axes
         .iter()
@@ -696,9 +690,8 @@ pub fn concat_pull_removable_axes(node: &Concat) -> Option<CircuitRc> {
             .filter(|i| **i < node.axis)
             .count();
     let new_node = Concat::new(
-        node.nodes
-            .iter()
-            .map(|node| remove_axes(node, &removable_non_axis).unwrap())
+        node.children()
+            .map(|node| remove_axes(&node, &removable_non_axis).unwrap())
             .collect(),
         new_axis,
         node.info().name,
@@ -714,18 +707,17 @@ pub fn concat_pull_removable_axes(node: &Concat) -> Option<CircuitRc> {
 pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     let mut did_anything = false;
     let mut new_args: EinsumArgs = einsum
-        .args
-        .iter()
+        .args()
         .map(|(node, ints)| {
-            let removable_axes = get_removable_axes(node);
+            let removable_axes = get_removable_axes(&node);
             if !removable_axes.is_empty() {
                 did_anything = true;
                 (
-                    remove_axes(node, &removable_axes).unwrap(),
+                    remove_axes(&node, &removable_axes).unwrap(),
                     filter_out_idx(ints, &removable_axes).into_iter().collect(),
                 )
             } else {
-                (node.clone(), ints.clone())
+                (node, ints.clone())
             }
         })
         .collect();
@@ -885,12 +877,11 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
 pub fn add_make_broadcasts_explicit(add: &Add) -> Option<Add> {
     let mut did_anything = false;
     let new_nodes = add
-        .nodes
-        .iter()
+        .children()
         .map(|node| {
             if node.info().shape[..] != add.info().shape[..] {
                 did_anything = true;
-                make_broadcast(node, &add.info().shape).unwrap()
+                make_broadcast(&node, &add.info().shape).unwrap()
             } else {
                 node.clone()
             }
@@ -925,7 +916,7 @@ pub fn distribute_once_raw(
         };
         call_on_sub(child_idx, einsum)
     };
-    match einsum.args[operand_idx].0.as_add() {
+    match einsum.children_sl()[operand_idx].as_add() {
         Some(add_node) => {
             let mut add = add_node.clone();
             if do_broadcasts && add.has_broadcast() {
@@ -934,18 +925,17 @@ pub fn distribute_once_raw(
             if add.has_broadcast() {
                 bail!(DistributeError::CouldntBroadcast { add, do_broadcasts });
             }
-            if add.nodes.len() == 0 {
+            if add.num_children() == 0 {
                 bail!(DistributeError::EmptyAddUnsupported {
                     einsum: einsum.clone(),
                     operand_idx
                 });
             }
             let summands = add
-                .nodes
-                .iter()
+                .children()
                 .enumerate()
                 .map(|(i, node)| {
-                    let mut new_args = einsum.args.clone();
+                    let mut new_args = einsum.args_cloned();
                     new_args[operand_idx].0 = node.clone();
                     on_sub(
                         i,
@@ -972,7 +962,7 @@ pub fn distribute_once(
     prefix: Option<String>,
     suffix: Option<String>,
 ) -> Result<Add> {
-    if operand_idx >= einsum.args.len() {
+    if operand_idx >= einsum.num_children() {
         bail!(DistributeError::OperandIdxTooLarge {
             einsum: einsum.clone(),
             operand_idx,
@@ -999,7 +989,7 @@ pub fn distribute_once_op(einsum: &Einsum, operand_idx: usize, do_broadcasts: bo
 /// not necessarily very useful?
 #[pyfunction]
 pub fn distribute_all(einsum: &Einsum) -> Option<Add> {
-    let add_op_idxs = filter_to_idx(&einsum.args, &|(node, _ints)| {
+    let add_op_idxs = filter_to_idx(einsum.args(), |(node, _ints)| {
         matches!(&***node, Circuit::Add(_)) && node.children().count() > 0
     });
     if add_op_idxs.is_empty() {
@@ -1023,15 +1013,14 @@ pub fn distribute_all(einsum: &Einsum) -> Option<Add> {
 pub fn einsum_of_permute_merge(einsum: &Einsum) -> Option<Einsum> {
     let mut did_anything = false;
     let new_args = einsum
-        .args
-        .iter()
+        .args()
         .map(|(node, ints)| {
             node.as_rearrange()
                 .and_then(|rearrange| {
                     rearrange.spec.get_fwd_permutation().map(|permutation| {
                         did_anything = true;
                         (
-                            rearrange.node.clone(),
+                            rearrange.node().clone(),
                             permutation.iter().map(|i| ints[*i]).collect(),
                         )
                     })
@@ -1052,9 +1041,9 @@ pub fn einsum_of_permute_merge(einsum: &Einsum) -> Option<Einsum> {
 #[pyfunction]
 pub fn permute_of_einsum_merge(rearrange: &Rearrange) -> Option<Einsum> {
     rearrange.spec.get_fwd_permutation().and_then(|perm| {
-        rearrange.node.as_einsum().map(|einsum| {
+        rearrange.node().as_einsum().map(|einsum| {
             Einsum::new(
-                einsum.args.clone(),
+                einsum.args_cloned(),
                 inverse_permutation(&perm)
                     .iter()
                     .map(|i| einsum.out_axes[*i])
@@ -1085,8 +1074,8 @@ pub fn einsum_merge_scalars_impl(einsum: &Einsum, only_one: bool) -> Option<Eins
     let mut num_scalars_found = 0;
     let mut scalar_mul: f64 = 1.0;
     let mut new_args: Vec<(CircuitRc, EinsumAxes)> = einsum
-        .args
-        .iter()
+        .args_cloned()
+        .into_iter()
         .filter(|(node, _ints)| match &***node {
             Circuit::Scalar(scalar) => {
                 if scalar.info().shape.is_empty() && (!only_one || scalar.value == 1.) {
@@ -1099,7 +1088,6 @@ pub fn einsum_merge_scalars_impl(einsum: &Einsum, only_one: bool) -> Option<Eins
             }
             _ => true,
         })
-        .cloned()
         .collect();
     if num_scalars_found == 0 || (num_scalars_found == 1 && scalar_mul != 1.) {
         return None;
@@ -1126,7 +1114,7 @@ pub fn einsum_remove_one(einsum: &Einsum) -> Option<Einsum> {
 pub fn einsum_concat_to_add(einsum: &Einsum) -> Option<Add> {
     // Which nodes contain a given einsum int, e.g. "ab,ab->" will produce {0: [0, 1], 1: [0, 1]}
     let mut nodes_by_einsum_dim = HashMap::default();
-    for (concat_idx, (_circ, einsum_ints)) in einsum.args.iter().enumerate() {
+    for (concat_idx, (_circ, einsum_ints)) in einsum.args().enumerate() {
         for einsum_int in einsum_ints {
             nodes_by_einsum_dim
                 .entry(einsum_int)
@@ -1135,12 +1123,11 @@ pub fn einsum_concat_to_add(einsum: &Einsum) -> Option<Add> {
         }
     }
 
-    let potential_concats_and_idx: Vec<(usize, &Concat)> = einsum
-        .args
-        .iter()
+    let potential_concats_and_idx: Vec<(usize, CircuitRc)> = einsum
+        .args()
         .enumerate()
         .filter_map(|(concat_idx, (circ, in_axes))| {
-            if let Circuit::Concat(concat) = &***circ {
+            if let Circuit::Concat(concat) = &**circ {
                 // `concat.axis` is in terms of this particular node, but we need the einsum dimension, so
                 // look it up, e.g. Einsum([0,1],[2,3] -> [0,3]), if the second input is a concat with
                 // concat.axis=1, needs to pull out 3.
@@ -1157,7 +1144,7 @@ pub fn einsum_concat_to_add(einsum: &Einsum) -> Option<Add> {
                 }
                 // This transformation only works if we're summing across the concat dimension.
                 if !einsum.out_axes.contains(&(concat_dim as u8)) {
-                    return Some((concat_idx, concat));
+                    return Some((concat_idx.clone(), circ));
                 }
             }
             None
@@ -1168,16 +1155,15 @@ pub fn einsum_concat_to_add(einsum: &Einsum) -> Option<Add> {
     }
 
     // TODO: Maybe make this work across multiple Concat nodes in an Einsum.
-    let (concat_idx, concat_node) = potential_concats_and_idx[0];
+    let (concat_idx, concat_node) = &potential_concats_and_idx[0];
 
     let inner_einsum: Vec<CircuitRc> = concat_node
-        .nodes
-        .iter()
+        .children()
         .map(|node| {
-            let mut new_args = einsum.args.clone();
+            let mut new_args = einsum.args_cloned();
             // Replace the concatted `a` dimension with the partial `a` we're mapping over, then
             // leave everything else in the output alone.
-            new_args[concat_idx] = (node.clone(), einsum.args[concat_idx].1.clone());
+            new_args[*concat_idx] = (node.clone(), einsum.in_axes[*concat_idx].clone());
             Einsum::nrc(new_args, einsum.out_axes.clone(), None)
         })
         .collect();
@@ -1188,7 +1174,7 @@ pub fn einsum_concat_to_add(einsum: &Einsum) -> Option<Add> {
 pub fn index_split_axes(node: &Index, top_axes: HashSet<usize>) -> Option<Index> {
     let mut bottom: Vec<TensorAxisIndex> = vec![];
     let mut top: Vec<TensorAxisIndex> = vec![];
-    if top_axes.iter().any(|i| *i >= node.node.info().rank()) {
+    if top_axes.iter().any(|i| *i >= node.node().info().rank()) {
         return None;
     }
     for (i, idx) in node.index.0.iter().enumerate() {
@@ -1210,7 +1196,7 @@ pub fn index_split_axes(node: &Index, top_axes: HashSet<usize>) -> Option<Index>
         }
     }
     Some(Index::new(
-        Index::nrc(node.node.clone(), TensorIndex(bottom), None),
+        Index::nrc(node.node(), TensorIndex(bottom), None),
         TensorIndex(top),
         None,
     ))
@@ -1218,18 +1204,12 @@ pub fn index_split_axes(node: &Index, top_axes: HashSet<usize>) -> Option<Index>
 
 #[pyfunction]
 pub fn rearrange_fuse(node: &Rearrange) -> Option<Rearrange> {
-    node.node.as_rearrange().and_then(|inner| {
+    node.node().as_rearrange().and_then(|inner| {
         RearrangeSpec::fuse(
             &inner.conform_to_input_shape_spec().canonicalize(true),
             &node.conform_to_input_shape_spec().canonicalize(true),
         )
-        .map(|spec| {
-            Rearrange::new(
-                inner.node.clone(),
-                spec.canonicalize(true),
-                node.info().name,
-            )
-        })
+        .map(|spec| Rearrange::new(inner.node(), spec.canonicalize(true), node.info().name))
         .ok()
     })
 }
@@ -1296,7 +1276,7 @@ pub fn push_down_index_raw(
         suffix
             .as_ref()
             .map(|s| {
-                node.node
+                node.node()
                     .info()
                     .name
                     .map(|inner_name| format!("{inner_name}_partial_{s}").into())
@@ -1337,7 +1317,7 @@ pub fn push_down_index_raw(
     // TODO: maybe handle symbolic axes? Possibilities:
     // Check we don't index into sum of symbolic axes
     // Handle rearrange case.
-    let out: CircuitRc = match &**node.node {
+    let out: CircuitRc = match &**node.node() {
         Circuit::Add(inner) => {
             let new_operands = inner
                 .nodes_and_rank_differences()
@@ -1417,13 +1397,18 @@ pub fn push_down_index_raw(
                         // should only be once match
                         if let Some(z) = ax {
                             index.0[inner.axis] = z.clone();
-                            return on_sub(i, inner.nodes[i].clone(), index, Some(out_name));
+                            return on_sub(
+                                i,
+                                inner.children_sl()[i].clone(),
+                                index,
+                                Some(out_name),
+                            );
                         }
                     }
                     unreachable!()
                 } else {
                     Concat::nrc(
-                        zip(inner.nodes.iter(), concat_indices)
+                        zip(inner.children(), concat_indices)
                             .enumerate()
                             .filter_map(|(i, (child, idx))| {
                                 idx.map(|idx| {
@@ -1440,12 +1425,9 @@ pub fn push_down_index_raw(
             } else {
                 let new_concat = Concat::nrc(
                     inner
-                        .nodes
-                        .iter()
+                        .children()
                         .enumerate()
-                        .map(|(i, operand)| {
-                            on_sub(i, operand.clone(), index_non_axis.clone(), None)
-                        })
+                        .map(|(i, operand)| on_sub(i, operand, index_non_axis.clone(), None))
                         .collect::<Result<_>>()?,
                     new_axis,
                     partial_name(),
@@ -1471,7 +1453,7 @@ pub fn push_down_index_raw(
             } else {
                 let rank_to_pass =
                     inner.info().rank() - inner.num_non_batchable_output_dims as usize;
-                let new_operands = zip(&inner.nodes, &inner.input_batchability)
+                let new_operands = zip(inner.children(), &inner.input_batchability)
                     .enumerate()
                     .map(|(i, (child, &batchable))| {
                         if batchable {
@@ -1483,9 +1465,9 @@ pub fn push_down_index_raw(
                                     .chain(vec![TensorAxisIndex::IDENT; ident_rank])
                                     .collect(),
                             );
-                            on_sub(i, child.clone(), index_passed, None)
+                            on_sub(i, child, index_passed, None)
                         } else {
-                            Ok(child.clone())
+                            Ok(child)
                         }
                     })
                     .collect::<Result<_>>()?;
@@ -1535,8 +1517,7 @@ pub fn push_down_index_raw(
             };
             let mut any_operand_index_nontrivial = false;
             let new_operands: Vec<(CircuitRc, EinsumAxes)> = inner
-                .args
-                .iter()
+                .args()
                 .enumerate()
                 .map(|(i, (node, ints))| {
                     let index_here = TensorIndex(
@@ -1554,8 +1535,8 @@ pub fn push_down_index_raw(
                         any_operand_index_nontrivial = true;
                     }
                     Ok((
-                        on_sub(i, node.clone(), index_here, None)?,
-                        ints_filter_index(ints), // todo make function work with sv
+                        on_sub(i, node, index_here, None)?,
+                        ints_filter_index(&ints), // todo make function work with sv
                     ))
                 })
                 .collect::<Result<_>>()?;
@@ -1605,7 +1586,7 @@ pub fn push_down_index_raw(
                         (
                             node.index.clone(),
                             inner.spec.clone(),
-                            inner.node.info().shape.clone(),
+                            inner.node().info().shape.clone(),
                             node.info().device_dtype.clone().unwrap_or_defaults().device,
                         ),
                         None,
@@ -1621,7 +1602,7 @@ pub fn push_down_index_raw(
                     spec: (*inner.spec).clone()
                 })
             };
-            let new_index = on_sub(0, inner.node.clone(), index, None)?;
+            let new_index = on_sub(0, inner.node().clone(), index, None)?;
             Rearrange::nrc_elim_identity(new_index, spec, out_name)
         }
         Circuit::Scatter(inner) => {
@@ -1652,7 +1633,7 @@ pub fn push_down_index_raw(
                     index_node: node.clone()
                 });
             }
-            let inner_node = on_sub(0, inner.node.clone(), TensorIndex(passed_index), None)?;
+            let inner_node = on_sub(0, inner.node().clone(), TensorIndex(passed_index), None)?;
 
             let new_scatter = Scatter::nrc(
                 inner_node,
@@ -1675,14 +1656,14 @@ pub fn push_down_index_raw(
             }
         }
         Circuit::Tag(tag) => Tag::nrc(
-            on_sub(0, tag.node.clone(), node.index.clone(), None)?,
+            on_sub(0, tag.node().clone(), node.index.clone(), None)?,
             tag.uuid,
             out_name,
         ),
         Circuit::DiscreteVar(var) => DiscreteVar::nrc(
             on_sub(
                 0,
-                var.values.clone(),
+                var.values().clone(),
                 TensorIndex(
                     iter::once(TensorAxisIndex::IDENT) // skip sample axis
                         .chain(node.index.0.iter().cloned())
@@ -1690,7 +1671,7 @@ pub fn push_down_index_raw(
                 ),
                 None,
             )?,
-            var.probs_and_group.clone(),
+            var.probs_and_group().clone(),
             out_name,
         ),
         Circuit::Scalar(_) => index_merge_scalar(node).unwrap(),
@@ -1700,8 +1681,7 @@ pub fn push_down_index_raw(
             let rank_to_pass = batch_shape.len();
 
             let new_nodes = m
-                .nodes
-                .iter()
+                .args()
                 .zip(&m.spec.arg_specs)
                 .enumerate()
                 .map(|(i, (input_node, arg_spec))| {
@@ -1766,10 +1746,10 @@ pub fn push_down_index_op(node: &Index) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn concat_repeat_to_rearrange(concat: &Concat) -> Option<Concat> {
-    if concat.nodes.is_empty() {
+    if concat.num_children() == 0 {
         return None;
     }
-    let mut prev_node: CircuitRc = concat.nodes[0].clone();
+    let mut prev_node: CircuitRc = concat.children_sl()[0].clone();
     let mut same_seen = 1;
     let mut did_anything = false;
     let mut new_nodes: Vec<CircuitRc> = vec![];
@@ -1778,12 +1758,11 @@ pub fn concat_repeat_to_rearrange(concat: &Concat) -> Option<Concat> {
     let extra_unique_node = Symbol::new_with_random_uuid(concat.info().shape.clone(), None).rc();
 
     for node in concat
-        .nodes
-        .iter()
+        .children()
         .dropping(1)
-        .chain(std::iter::once(&extra_unique_node))
+        .chain(std::iter::once(extra_unique_node))
     {
-        if node != &prev_node {
+        if node != prev_node {
             if same_seen > 1 {
                 did_anything = true;
                 let mut int_sizes = prev_node.info().shape.clone();
@@ -1899,7 +1878,7 @@ pub fn add_outer_product_broadcasts_on_top(add: &Add) -> Option<Add> {
         .map(|operand_set_bitmask| {
             Add::nrc(
                 BitIter::from(*operand_set_bitmask)
-                    .map(|i| add.nodes[i].clone())
+                    .map(|i| add.children_sl()[i].clone())
                     .collect(),
                 None,
             )
@@ -1913,8 +1892,8 @@ pub fn add_outer_product_broadcasts_on_top(add: &Add) -> Option<Add> {
 pub fn extract_add(add: &Add, sub: &Add) -> Option<Add> {
     let mut counts = add.to_counts();
 
-    for item in &sub.nodes {
-        match counts.entry(item.clone()) {
+    for item in sub.children() {
+        match counts.entry(item) {
             Entry::Occupied(mut entry) => {
                 let new = entry.get().saturating_sub(1);
                 if new == 0 {
@@ -1947,10 +1926,8 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                 let mut scalar: f64 = 1.0;
                 let mut new_name: Option<Name> = None;
                 let no_scalar_children: EinsumArgs = e
-                    .args
-                    .iter()
-                    .filter(|args| {
-                        let (child, _axes) = &**args;
+                    .args_cloned().into_iter()
+                    .filter(|(child, _)| {
                         if let Some(x) = child.as_scalar() && x.info().shape.len() == 0  {
                             scalar *= x.value;
                             reduced_any += 1;
@@ -1968,7 +1945,6 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                             true
                         }
                     })
-                    .cloned()
                     .collect();
 
                 (
@@ -2001,7 +1977,7 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
         }
     }
 
-    if hm.len() == add.nodes.len() {
+    if hm.len() == add.num_children() {
         return None;
     }
 
@@ -2012,8 +1988,8 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                 einsum.rc()
             } else {
                 let name = einsum.info().name;
-                let out_axes = einsum.out_axes;
-                let mut v = einsum.args;
+                let out_axes = einsum.out_axes.clone();
+                let mut v = einsum.args_cloned();
                 v.push((Scalar::new(scalar, sv![], scalar_name).rc(), tu8v![]));
                 Einsum::nrc(v, out_axes, name)
             }
@@ -2025,16 +2001,16 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
 
 #[pyfunction]
 pub fn einsum_permute_to_rearrange(einsum: &Einsum) -> Option<Rearrange> {
-    if einsum.args.len() == 1
+    if einsum.num_children() == 1
         && einsum.out_axes.len() > 0
-        && einsum.args[0].1.len() == einsum.out_axes.len()
+        && einsum.in_axes[0].len() == einsum.out_axes.len()
         && is_unique(&einsum.out_axes)
-        && is_unique(&einsum.args[0].1)
+        && is_unique(&einsum.in_axes[0])
     {
         return Some(Rearrange::new(
-            einsum.args[0].0.clone(),
+            einsum.children_sl()[0].clone(),
             RearrangeSpec::new(
-                einsum.args[0].1.iter().map(|z| tu8v![*z]).collect(),
+                einsum.in_axes[0].iter().map(|z| tu8v![*z]).collect(),
                 einsum.out_axes.iter().map(|z| tu8v![*z]).collect(),
                 sv![OpSize::NONE;(*einsum.out_axes.iter().max().unwrap()+1) as usize],
             )
