@@ -16,7 +16,7 @@ pub fn strip_names_and_tags(circuit: CircuitRc, context: &mut OptimizationContex
         circuit.clone(),
         &|circuit, _| {
             if let Some(tag) = circuit.as_tag() {
-                return Some(tag.node.clone());
+                return Some(tag.node().clone());
             }
             if circuit.info().name.is_some() {
                 if circuit.is_irreducible_node() {
