@@ -18,10 +18,10 @@ use crate::algebraic_rewrite::{get_removable_axes, remove_axes};
 #[pyfunction]
 pub fn scatter_fuse(scatter: &Scatter) -> Option<Scatter> {
     // this is just composing indices, lower is top
-    if let Circuit::Scatter(inner) = &**scatter.node {
+    if let Circuit::Scatter(inner) = &**scatter.node() {
         let index_composed = compose(&inner.index, &scatter.index)?;
         Some(Scatter::new(
-            inner.node.clone(),
+            inner.node().clone(),
             index_composed,
             scatter.info().shape.clone(),
             scatter.info().name,
@@ -49,11 +49,11 @@ pub fn einsum_pull_scatter(einsum: &Einsum) -> Option<CircuitRc> {
     let mut did_anything = false;
     let int_sizes = einsum.shape_map().unwrap();
     let mut int_slices: HashMap<u8, USlice> = HashMap::default();
-    for (node, ints) in &einsum.args {
-        match &***node {
+    for (node, ints) in einsum.args() {
+        match &**node {
             Circuit::Scatter(scatter) => {
                 did_anything = true;
-                for (slice, i) in zip(scatter.index.all_uslices().unwrap(), ints) {
+                for (slice, i) in zip(scatter.index.all_uslices().unwrap(), ints.iter()) {
                     int_slices.insert(
                         *i,
                         int_slices
@@ -78,20 +78,19 @@ pub fn einsum_pull_scatter(einsum: &Einsum) -> Option<CircuitRc> {
         Some(Scalar::new(0.0, einsum.info().shape.clone(), None).rc())
     } else {
         let new_args = einsum
-            .args
-            .iter()
-            .map(|(node, ints)| match &***node {
+            .args()
+            .map(|(node, ints)| match &**node {
                 Circuit::Scatter(inner) => {
                     let index_orig_base = ints.iter().map(|i| int_slices[i]).collect();
                     let index =
                         uslices_shrink_base(&index_orig_base, &inner.index.all_uslices().unwrap());
                     (
-                        Index::nrc(inner.node.clone(), uslices_to_index(&index), None),
+                        Index::nrc(inner.node().clone(), uslices_to_index(&index), None),
                         ints.clone(),
                     )
                 }
                 _ => (
-                    Index::nrc(node.clone(), uslice_map_get_index(ints, &int_slices), None),
+                    Index::nrc(node.clone(), uslice_map_get_index(&ints, &int_slices), None),
                     ints.clone(),
                 ),
             })
@@ -108,11 +107,7 @@ pub fn einsum_pull_scatter(einsum: &Einsum) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn add_pull_scatter(add: &Add) -> Option<Scatter> {
-    if !add
-        .nodes
-        .iter()
-        .all(|x| matches!(&***x, Circuit::Scatter(_)))
-    {
+    if !add.children().all(|x| matches!(**x, Circuit::Scatter(_))) {
         return None;
     }
     let mut slices: Vec<USlice> = add
@@ -179,7 +174,7 @@ pub fn add_pull_scatter(add: &Add) -> Option<Scatter> {
 #[pyfunction]
 pub fn scatter_elim_identity(scatter: &Scatter) -> Option<CircuitRc> {
     if scatter.is_identity() {
-        Some(scatter.node.clone())
+        Some(scatter.node().clone())
     } else {
         None
     }
@@ -188,7 +183,7 @@ pub fn scatter_elim_identity(scatter: &Scatter) -> Option<CircuitRc> {
 #[pyfunction]
 pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
     let mut did_anything = false;
-    if let Circuit::Einsum(inner) = &**node.node {
+    if let Circuit::Einsum(inner) = &**node.node() {
         let mut int_slices: HashMap<u8, USlice> = HashMap::default();
         let canon_index = node.index.canonicalize(&inner.info().shape);
         let containing_uslices: Vec<Option<USlice>> = canon_index
@@ -220,11 +215,10 @@ pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
             Some(Scalar::new(0.0, node.info().shape.clone(), None).rc())
         } else {
             let new_args = inner
-                .args
-                .iter()
+                .args()
                 .map(|(node, ints)| {
                     (
-                        Index::nrc(node.clone(), uslice_map_get_index(ints, &int_slices), None),
+                        Index::nrc(node, uslice_map_get_index(&ints, &int_slices), None),
                         ints.clone(),
                     )
                 })
@@ -275,7 +269,7 @@ pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn scatter_pull_removable_axes(scatter: &Scatter) -> Option<Rearrange> {
-    let removable_axes = get_removable_axes(&scatter.node);
+    let removable_axes = get_removable_axes(&scatter.node());
     let scatter_uslices = scatter.index.all_uslices().unwrap();
     let removable_axes: HashSet<usize> = removable_axes
         .iter()
@@ -291,7 +285,7 @@ pub fn scatter_pull_removable_axes(scatter: &Scatter) -> Option<Rearrange> {
     if removable_axes.is_empty() {
         None
     } else {
-        let new_inner = remove_axes(&scatter.node, &removable_axes).unwrap();
+        let new_inner = remove_axes(&scatter.node(), &removable_axes).unwrap();
         Some(Rearrange::new(
             Scatter::nrc(
                 new_inner,
@@ -312,7 +306,7 @@ pub fn scatter_pull_removable_axes(scatter: &Scatter) -> Option<Rearrange> {
 
 #[pyfunction]
 pub fn scatter_to_concat(scatter: &Scatter) -> CircuitRc {
-    let mut result = scatter.node.clone();
+    let mut result = scatter.node().clone();
     for (i, (idx, l)) in
         zip(scatter.index.all_uslices().unwrap(), &scatter.info().shape).enumerate()
     {
@@ -336,10 +330,9 @@ pub fn scatter_to_concat(scatter: &Scatter) -> CircuitRc {
 #[pyfunction]
 pub fn concat_to_scatter(concat: &Concat) -> Option<Scatter> {
     let pre_zeros = concat
-        .nodes
-        .iter()
+        .children()
         .take_while(|node| {
-            if let Circuit::Scalar(sc) = &****node && sc.value==0.0{
+            if let Circuit::Scalar(sc) = &***node && sc.value==0.0{
             true
         }else{
             false
@@ -347,19 +340,18 @@ pub fn concat_to_scatter(concat: &Concat) -> Option<Scatter> {
         })
         .count();
     let post_zeros = concat
-        .nodes
-        .iter()
+        .children()
         .rev()
         .take_while(|node| {
-            if let Circuit::Scalar(sc) = &****node && sc.value==0.0{
+            if let Circuit::Scalar(sc) = &***node && sc.value==0.0{
             true
         }else{
             false
         }
         })
         .count();
-    let end = concat.nodes.len() - post_zeros;
-    if pre_zeros == 0 && post_zeros == 0 || (pre_zeros + post_zeros >= concat.nodes.len()) {
+    let end = concat.num_children() - post_zeros;
+    if pre_zeros == 0 && post_zeros == 0 || (pre_zeros + post_zeros >= concat.num_children()) {
         return None;
     }
     let starts = cumsum(&concat.get_sizes_at_axis());
@@ -367,7 +359,7 @@ pub fn concat_to_scatter(concat: &Concat) -> Option<Scatter> {
     let scatter_index = TensorIndex::new_single(cslice, concat.axis, concat.info().rank())
         .canonicalize(&concat.info().shape);
     let new_concat = Concat::nrc(
-        concat.nodes[pre_zeros..end].to_vec(),
+        concat.children_sl()[pre_zeros..end].to_vec(),
         concat.axis,
         concat.info().name,
     );
