@@ -8,6 +8,7 @@ use circuit_base::{
     cached_circuit_properties::max_non_leaf_size,
     circuit_node_private::CircuitNodePrivate,
     circuit_utils::{hash_to_node_non_free, is_definitely_view_on_child, toposort_circuit},
+    flat_concat,
     module::{any_children_with_symbolic_sizes, substitute_all_modules},
     prelude::*,
     print::{oom_fmt, PrintOptions},
@@ -222,6 +223,7 @@ impl Schedule {
     }
     #[pyo3(signature=(settings = Default::default()))]
     pub fn evaluate(&self, settings: OptimizationSettings) -> Result<Tensor> {
+        let timed = settings.verbose >= 1;
         let eval = |timed| -> Result<_> {
             let result = if settings.adjust_numerical_scale {
                 evaluate_schedule_adjust_numerical_scale(self, settings)?
@@ -244,7 +246,6 @@ impl Schedule {
             }
             Ok(result)
         };
-        let timed = settings.verbose >= 1;
         Ok(timed!(eval(timed)?, 10, timed))
     }
 
@@ -1025,6 +1026,35 @@ pub fn scheduled_evaluate(circuit: CircuitRc, settings: OptimizationSettings) ->
     Ok(evaluate_schedule(&schedule)?[&schedule.output_circuit.unwrap().0].clone())
 }
 
+#[pyfunction]
+#[pyo3(name = "circuit_to_schedule")]
+pub fn py_circuit_to_schedule(
+    circuit: CircuitRc,
+    settings: OptimizationSettings,
+) -> Result<Schedule> {
+    circuit_to_schedule(
+        circuit.clone(),
+        &mut OptimizationContext::new_settings(settings),
+    )
+}
+
+#[pyfunction]
+#[pyo3(name = "circuit_to_schedule_many")]
+pub fn py_circuit_to_schedule_many(
+    circuits: Vec<CircuitRc>,
+    settings: OptimizationSettings,
+) -> Result<Schedule> {
+    let flat_concatted = flat_concat(circuits.clone())?.rc();
+    let mut schedule = py_circuit_to_schedule(flat_concatted, settings)?;
+    schedule.split_shapes = Some(
+        circuits
+            .iter()
+            .map(|x| x.info().shape.clone())
+            .collect::<Vec<Shape>>(),
+    );
+    Ok(schedule)
+}
+
 fn schedule_module_spec(spec: &ModuleSpec, context: &mut OptimizationContext) -> Schedule {
     context
         .cache
@@ -1066,7 +1096,7 @@ fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationConte
                         .expect("unused args removed at start of circuit_to_schedule!");
 
                     let original_children: Vec<usize> =
-                        mn.nodes.iter().cloned().map(get_child_key).collect();
+                        mn.args().cloned().map(get_child_key).collect();
 
                     let inner_schedule = schedule_module_spec(&mn.spec, context);
                     let mut inner_to_outer_key = HashMap::default();
