@@ -607,13 +607,13 @@ pub struct IterateMatchResults {
 #[pymethods]
 impl IterateMatchResults {
     #[new]
-    #[args(updated = "None", found = "false")]
+    #[pyo3(signature=(updated = None, found = false))]
     fn new(updated: Option<UpdatedIterativeMatcher>, found: bool) -> Self {
         Self { updated, found }
     }
 
     #[staticmethod]
-    #[args(found = "false")]
+    #[pyo3(signature=(found = false))]
     pub fn new_finished(found: bool) -> Self {
         Self {
             updated: Some(None.into()),
@@ -675,7 +675,7 @@ impl IterativeMatcher {
 #[pymethods]
 impl IterativeMatcher {
     #[new]
-    #[args(inps = "*")]
+    #[pyo3(signature=(*inps))]
     // also py_new
     fn special_case_any(inps: Vec<IterativeMatcherRc>) -> Self {
         match inps.into() {
@@ -691,7 +691,7 @@ impl IterativeMatcher {
     }
 
     #[staticmethod]
-    #[args(match_next = "false")]
+    #[pyo3(signature=(match_next = false))]
     pub fn term(match_next: bool) -> Self {
         IterativeMatcherData::Term(match_next).into()
     }
@@ -1021,7 +1021,7 @@ impl IterativeMatcher {
     }
 
     #[staticmethod]
-    #[args(matchers = "*")]
+    #[pyo3(signature = (*matchers))]
     pub fn any(matchers: Vec<IterativeMatcherRc>) -> Self {
         IterativeMatcherData::Chains(
             matchers
@@ -1033,13 +1033,13 @@ impl IterativeMatcher {
     }
 
     #[staticmethod]
-    #[args(matchers = "*")]
+    #[pyo3(signature = (*matchers))]
     pub fn all(matchers: Vec<IterativeMatcherRc>) -> Self {
         IterativeMatcherData::All(matchers).into()
     }
 
     #[staticmethod]
-    #[args(first, rest = "*", must_be_sub = "false")]
+    #[pyo3(signature=(first, *rest, must_be_sub = false))]
     pub fn new_chain(
         first: IterativeMatcherRc,
         rest: Vec<IterativeMatcherRc>,
@@ -1049,7 +1049,7 @@ impl IterativeMatcher {
     }
 
     #[staticmethod]
-    #[args(chains = "*", must_be_sub = "false")]
+    #[pyo3(signature=(*chains, must_be_sub = false))]
     pub fn new_chain_many(chains: Vec<Vec<IterativeMatcherRc>>, must_be_sub: bool) -> Result<Self> {
         Ok(IterativeMatcherData::Chains(
             chains
@@ -1106,7 +1106,7 @@ impl IterativeMatcher {
         IterativeMatcherData::PyFunc(f).into()
     }
 
-    #[args(others = "*")]
+    #[pyo3(signature=(*others))]
     pub fn new_or(&self, others: Vec<IterativeMatcherRc>) -> Self {
         Self::any([self.clone().into()].into_iter().chain(others).collect())
     }
@@ -1177,12 +1177,12 @@ macro_rules! dup_functions {
 #[apply(dup_functions)]
 #[self_id(self_)]
 impl IterativeMatcher {
-    #[args(
-        term_if_matches = "false",
-        start_depth = "None",
-        end_depth = "None",
-        term_early_at = "MatcherFromPyBase::Always(false).into()"
-    )]
+    #[pyo3(signature=(
+        term_if_matches = false,
+        start_depth = None,
+        end_depth = None,
+        term_early_at = MatcherFromPyBase::Always(false).into()
+    ))]
     pub fn filter(
         &self,
         term_if_matches: bool,
@@ -1200,11 +1200,11 @@ impl IterativeMatcher {
         )
     }
 
-    #[args(
-        term_if_matches = "false",
-        depth_slice = "Slice::IDENT",
-        term_early_at = "MatcherFromPyBase::Always(false).into()"
-    )]
+    #[pyo3(signature=(
+        term_if_matches = false,
+        depth_slice = Slice::IDENT,
+        term_early_at = MatcherFromPyBase::Always(false).into()
+    ))]
     pub fn filter_sl(
         &self,
         term_if_matches: bool,
@@ -1220,7 +1220,7 @@ impl IterativeMatcher {
         )
     }
 
-    #[args(rest = "*", must_be_sub = "false")]
+    #[pyo3(signature=(*rest, must_be_sub = false))]
     pub fn chain(&self, rest: Vec<IterativeMatcherRc>, must_be_sub: bool) -> IterativeMatcher {
         // TODO: flatten
         Self::new_chain_many(
@@ -1230,7 +1230,7 @@ impl IterativeMatcher {
         .unwrap()
     }
 
-    #[args(rest = "*", must_be_sub = "false")]
+    #[pyo3(signature=(*rest, must_be_sub = false))]
     pub fn chain_many(
         &self,
         rest: Vec<Vec<IterativeMatcherRc>>,
@@ -1266,7 +1266,7 @@ impl IterativeMatcher {
         IterativeMatcherData::SpecCircuit(self_.clone().rc()).into()
     }
 
-    #[args(enable = "true")]
+    #[pyo3(signature=(enable = true))]
     pub fn filter_module_spec(&self, enable: bool) -> IterativeMatcher {
         if enable {
             IterativeMatcherData::NoModuleSpec(self_.clone().rc()).into()
@@ -1275,12 +1275,12 @@ impl IterativeMatcher {
         }
     }
 
-    #[args(fancy_validate = "Getter::default().default_fancy_validate")]
+    #[pyo3(signature=(circuit, fancy_validate = Getter::default().default_fancy_validate))]
     pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> Result<BTreeSet<CircuitRc>> {
         Getter::default().get(circuit, self_.crc(), Some(fancy_validate))
     }
 
-    #[args(fancy_validate = "Getter::default().default_fancy_validate")]
+    #[pyo3(signature=(circuit, fancy_validate = Getter::default().default_fancy_validate))]
     pub fn get_unique_op(
         &self,
         circuit: CircuitRc,
@@ -1289,7 +1289,7 @@ impl IterativeMatcher {
         Getter::default().get_unique_op(circuit, self_.crc(), Some(fancy_validate))
     }
 
-    #[args(fancy_validate = "Getter::default().default_fancy_validate")]
+    #[pyo3(signature=(circuit, fancy_validate = Getter::default().default_fancy_validate))]
     pub fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> Result<CircuitRc> {
         Getter::default().get_unique(circuit, self_.crc(), Some(fancy_validate))
     }
@@ -1311,7 +1311,7 @@ impl IterativeMatcher {
         Getter::default().validate(circuit, self_.crc())
     }
 
-    #[args(default_fancy_validate = "Getter::default().default_fancy_validate")]
+    #[pyo3(signature=(default_fancy_validate = Getter::default().default_fancy_validate))]
     pub fn getter(&self, default_fancy_validate: bool) -> BoundGetter {
         Getter::new(default_fancy_validate).bind(self_.crc())
     }
@@ -1323,11 +1323,13 @@ impl IterativeMatcher {
         AnyFound::new().bind(self_.crc())
     }
 
-    #[args(
-        cache_transform = "Updater::default().cache_transform",
-        cache_update = "Updater::default().cache_update",
-        fancy_validate = "Updater::default().default_fancy_validate"
-    )]
+    #[pyo3(signature=(
+        circuit,
+        transform,
+        cache_transform = Updater::default().cache_transform,
+        cache_update = Updater::default().cache_update,
+        fancy_validate = Updater::default().default_fancy_validate
+    ))]
     pub fn update(
         &self,
         circuit: CircuitRc,
@@ -1343,11 +1345,12 @@ impl IterativeMatcher {
         )
     }
 
-    #[args(
-        cache_transform = "Updater::default().cache_transform",
-        cache_update = "Updater::default().cache_update",
-        default_fancy_validate = "Updater::default().default_fancy_validate"
-    )]
+    #[pyo3(signature=(
+        transform,
+        cache_transform = Updater::default().cache_transform,
+        cache_update = Updater::default().cache_update,
+        default_fancy_validate = Updater::default().default_fancy_validate
+    ))]
     pub fn updater(
         &self,
         transform: TransformRc,
@@ -1410,12 +1413,14 @@ pub enum IterativeMatcherError {
     ExpectedModule { circuit_type: CircuitType },
 }
 
-#[pyfunction(
-    term_if_matches = "false",
-    start_depth = "None",
-    end_depth = "None",
-    term_early_at = "MatcherFromPyBase::Always(false).into()"
-)]
+#[pyfunction]
+#[pyo3(signature=(
+    matcher,
+    term_if_matches = false,
+    start_depth = None,
+    end_depth = None,
+    term_early_at = MatcherFromPyBase::Always(false).into()
+))]
 pub fn restrict(
     matcher: IterativeMatcherRc,
     term_if_matches: bool,
@@ -1434,11 +1439,13 @@ pub fn restrict(
     .into()
 }
 
-#[pyfunction(
-    term_if_matches = "false",
-    depth_slice = "Slice::IDENT",
-    term_early_at = "MatcherFromPyBase::Always(false).into()"
-)]
+#[pyfunction]
+#[pyo3(signature=(
+    matcher,
+    term_if_matches = false,
+    depth_slice = Slice::IDENT,
+    term_early_at = MatcherFromPyBase::Always(false).into()
+))]
 pub fn restrict_sl(
     matcher: IterativeMatcherRc,
     term_if_matches: bool,
@@ -1453,11 +1460,12 @@ pub fn restrict_sl(
     ))
 }
 
-#[pyfunction(
-    start_depth = "None",
-    end_depth = "None",
-    term_early_at = "MatcherFromPyBase::Always(false).into()"
-)]
+#[pyfunction]
+#[pyo3(signature=(
+    start_depth = None,
+    end_depth = None,
+    term_early_at = MatcherFromPyBase::Always(false).into()
+))]
 pub fn new_traversal(
     start_depth: Option<u32>,
     end_depth: Option<u32>,
