@@ -115,7 +115,7 @@ impl RegexWrap {}
 #[pymethods]
 impl RegexWrap {
     #[new]
-    #[args(escape_dot = "true")]
+    #[pyo3(signature=(pattern, escape_dot = true))]
     pub fn new(pattern: String, escape_dot: bool) -> Result<Self> {
         let new_pattern = if escape_dot {
             Regex::new(r"(\\\.)|(\.)")
@@ -333,7 +333,7 @@ impl Matcher {
 #[pymethods]
 impl Matcher {
     #[new]
-    #[args(inps = "*")]
+    #[pyo3(signature=(*inps))]
     fn py_new(inps: Vec<MatcherRc>) -> Self {
         match inps.into() {
             ESM::Empty => MatcherData::Always(false).into(),
@@ -432,7 +432,7 @@ impl Matcher {
     }
 
     #[staticmethod]
-    #[args(escape_dot = "true")]
+    #[pyo3(signature=(pattern, escape_dot = true))]
     pub fn regex(pattern: String, escape_dot: bool) -> Result<Self> {
         Ok(MatcherData::Regex(RegexWrap::new(pattern, escape_dot)?).into())
     }
@@ -457,25 +457,25 @@ impl Matcher {
     }
 
     #[staticmethod]
-    #[args(types = "*")]
+    #[pyo3(signature=(*types))]
     pub fn types(types: Vec<CircuitType>) -> Self {
         MatcherData::Type(types.into_iter().collect::<BTreeSet<_>>()).into()
     }
 
     #[staticmethod]
-    #[args(circuits = "*")]
+    #[pyo3(signature=(*circuits))]
     pub fn circuits(circuits: Vec<CircuitRc>) -> Self {
         MatcherData::EqM(circuits.into_iter().collect::<BTreeSet<_>>()).into()
     }
 
     #[staticmethod]
-    #[args(matchers = "*")]
+    #[pyo3(signature=(*matchers))]
     pub fn all(matchers: Vec<MatcherRc>) -> Self {
         MatcherData::All(matchers).into()
     }
 
     #[staticmethod]
-    #[args(matchers = "*")]
+    #[pyo3(signature=(*matchers))]
     pub fn any(matchers: Vec<MatcherRc>) -> Self {
         MatcherData::Any(matchers).into()
     }
@@ -485,12 +485,12 @@ impl Matcher {
         MatcherData::Not(self.crc()).into()
     }
 
-    #[args(others = "*")]
+    #[pyo3(signature=(*others))]
     pub fn new_and(&self, others: Vec<MatcherRc>) -> Self {
         Self::all([self.crc()].into_iter().chain(others).collect())
     }
 
-    #[args(others = "*")]
+    #[pyo3(signature=(*others))]
     pub fn new_or(&self, others: Vec<MatcherRc>) -> Self {
         Self::any([self.crc()].into_iter().chain(others).collect())
     }
