@@ -349,7 +349,8 @@ impl Matcher {
 //     ))
 // }
 
-#[pyfunction(print_options = "Default::default()")]
+#[pyfunction]
+#[pyo3(signature=(circuit, matcher, print_options = Default::default()))]
 pub fn repr_matcher_debug(
     circuit: CircuitRc,
     matcher: IterativeMatcherRc,
@@ -437,7 +438,8 @@ pub fn print_matcher_debug(
     Ok(())
 }
 
-#[pyfunction(discard_old_name = "false")]
+#[pyfunction]
+#[pyo3(signature=(circuit, matcher, discard_old_name = false))]
 pub fn append_matchers_to_names(
     circuit: CircuitRc,
     matcher: IterativeMatcherRc,
