@@ -252,12 +252,12 @@ impl Default for RandomSampleSpec {
 
 impl SampleSpecTrait for RandomSampleSpec {
     fn sample_var(&self, var: &DiscreteVar) -> Result<CircuitRc> {
-        let index = self.sample_impl(var.probs_and_group.clone())?;
+        let index = self.sample_impl(var.probs_and_group().clone())?;
         let index = if self.simplify
             && index.info().is_explicitly_computable()
-            && var.probs_and_group.is_tag()
-            && let Circuit::Tag(c) = &**var.probs_and_group
-            && c.node.is_leaf_constant()
+            && var.probs_and_group().is_tag()
+            && let Circuit::Tag(c) = &***var.probs_and_group()
+            && c.node().is_leaf_constant()
         {
             evaluate(index.clone())
                 .and_then(|x| Ok(Array::try_new(x, index.info().name)?.rc()))
@@ -268,9 +268,9 @@ impl SampleSpecTrait for RandomSampleSpec {
         let batch_x = true;
         let batch_index = false;
         let res = GeneralFunction::gen_index(
-            var.values.clone(),
+            var.values().clone(),
             index,
-            -(var.values.ndim() as i64),
+            -(var.values().ndim() as i64),
             batch_x,
             batch_index,
             true,
@@ -373,7 +373,7 @@ impl RunDiscreteVarAllSpec {
         let mut add_all_groups = |circ: CircuitRc| {
             visit_circuit(circ, |sub| {
                 if let Some(var) = sub.as_discrete_var() {
-                    discrete_vars_groups.insert(var.probs_and_group.clone());
+                    discrete_vars_groups.insert(var.probs_and_group().clone());
                 } else if sub.is_var() {
                     bail!(SampleError::UnhandledVarError { circ: sub })
                 }
@@ -390,11 +390,7 @@ impl RunDiscreteVarAllSpec {
 
 impl SampleSpecTrait for RunDiscreteVarAllSpec {
     fn sample_var(&self, var: &DiscreteVar) -> CircResult {
-        let this_group_i = match self
-            .groups
-            .iter()
-            .position(|r| r.clone() == var.probs_and_group)
-        {
+        let this_group_i = match self.groups.iter().position(|r| r == var.probs_and_group()) {
             Some(g) => g,
             None => {
                 return Err(Error::from(SampleError::UnhandledVarError {
@@ -409,7 +405,7 @@ impl SampleSpecTrait for RunDiscreteVarAllSpec {
             .collect_vec();
 
         let mut sampled_values = Index::nrc(
-            var.values.clone(),
+            var.values().clone(),
             TensorIndex(vec![TensorAxisIndex::Slice(self.subsets[this_group_i])]),
             None,
         );
@@ -418,7 +414,7 @@ impl SampleSpecTrait for RunDiscreteVarAllSpec {
             let counts = self
                 .groups
                 .iter()
-                .filter(|&g| g != &var.probs_and_group)
+                .filter(|&g| g != var.probs_and_group())
                 .map(|g| g.info().shape[0])
                 .collect();
             sampled_values = sampled_values
@@ -628,7 +624,7 @@ impl Sampler {
         cum: &Cumulant,
         matcher: IterativeMatcherRc,
     ) -> Result<CircuitRc> {
-        match cum.nodes.len() {
+        match cum.num_children() {
             0 => Ok(Scalar::new(1., Sv::new(), None).rc()),
             1 => {
                 let sampled = self.sample(cum.children().next().unwrap())?;
@@ -747,10 +743,7 @@ pub fn factored_cumulant_expectation_rewrite(cumulant: &Cumulant) -> CircuitRc {
 
         let permutation = dim_permutation_for_circuits(
             p,
-            cumulants
-                .iter()
-                .map(|c| c.children().collect_vec())
-                .collect_vec(),
+            cumulants.iter().map(|c| c.children_sl()).collect_vec(),
             children.len(),
         );
 
