@@ -86,10 +86,10 @@ impl Transform {
         TransformData::Ident.into()
     }
 
-    #[args(
-        cache_transform = "Updater::default().cache_transform",
-        cache_update = "Updater::default().cache_update"
-    )]
+    #[pyo3(signature=(
+        cache_transform = Updater::default().cache_transform,
+        cache_update = Updater::default().cache_update
+    ))]
     fn updater(&self, cache_transform: bool, cache_update: bool) -> Updater {
         Updater {
             transform: self.clone().into(),
