@@ -17,11 +17,13 @@ use rr_util::{
 };
 use rustc_hash::FxHashMap as HashMap;
 use uuid::uuid;
-#[pyfunction(
-    hash_child_count = "true",
-    hash_child_shapes = "false",
-    hash_name = "true"
-)]
+#[pyfunction]
+#[pyo3(signature=(
+    circuit,
+    hash_child_count = true,
+    hash_child_shapes = false,
+    hash_name = true
+))]
 pub fn compute_self_hash(
     circuit: CircuitRc,
     hash_child_count: bool,
@@ -54,17 +56,20 @@ pub fn compute_self_hash(
     m.finalize().into()
 }
 
-#[pyfunction(
-    options = "Default::default()",
-    require_child_count_same = "true",
-    require_child_shapes_same = "false",
-    require_name_same = "true",
-    print_legend = "true",
-    same_self_color = "clicolor!(Blue)",
-    same_color = "CliColor::NONE",
-    new_color = "clicolor!(Green)",
-    removed_color = "clicolor!(Red)"
-)]
+#[pyfunction]
+#[pyo3(signature=(
+    new,
+    old,
+    options = Default::default(),
+    require_child_count_same = true,
+    require_child_shapes_same = false,
+    require_name_same = true,
+    print_legend = true,
+    same_self_color = clicolor!(Blue),
+    same_color = CliColor::NONE,
+    new_color = clicolor!(Green),
+    removed_color = clicolor!(Red)
+))]
 pub fn diff_circuits(
     new: CircuitRc,
     old: CircuitRc,
