@@ -42,7 +42,7 @@ pub fn distribute(
     #[key((einsum.info().hash, traversal.clone()), (HashBytes, IterativeMatcherRc))]
     #[use_try]
     fn distribute_rec(einsum: Einsum, traversal: IterativeMatcherRc) -> Result<CircuitRc> {
-        if operand_idx >= einsum.args.len() {
+        if operand_idx >= einsum.num_children() {
             if allow_partial_distribute {
                 return Ok(einsum.rc());
             } else {
@@ -53,7 +53,7 @@ pub fn distribute(
             };
         }
         let updated = traversal
-            .match_iterate(einsum.args[operand_idx].0.clone())?
+            .match_iterate(einsum.children_sl()[operand_idx].clone())?
             .unwrap_or_same(traversal)
             .0;
 
@@ -62,7 +62,7 @@ pub fn distribute(
         }
 
         let traversal_per_child =
-            updated.per_child_with_term(einsum.args[operand_idx].0.num_children());
+            updated.per_child_with_term(einsum.children_sl()[operand_idx].num_children());
 
         let distributed = distribute_once_raw(
             &einsum,
