@@ -43,7 +43,7 @@ pub fn push_down_index(
             }
         }
         let updated = traversal
-            .match_iterate(index.node.clone())?
+            .match_iterate(index.node().clone())?
             .unwrap_or_same(traversal)
             .0;
 
@@ -51,7 +51,7 @@ pub fn push_down_index(
             return Ok(index.rc());
         }
 
-        let traversal_per_child = updated.per_child_with_term(index.node.num_children());
+        let traversal_per_child = updated.per_child_with_term(index.node().num_children());
 
         push_down_index_raw(
             &index,
