@@ -335,7 +335,7 @@ fn test_u8set() {
     assert!(u8s.len() == 4);
     dbg!(u8s.into_iter().collect::<Vec<u8>>(), u8s.len());
     assert!(u8s.contains(150) && !u8s.contains(1));
-    let mut u8s2: U8Set = vec![0, 1, 2, 2, 50].into_iter().collect();
+    let u8s2: U8Set = vec![0, 1, 2, 2, 50].into_iter().collect();
     assert!(u8s2.union(&u8s).into_iter().collect::<Vec<u8>>() == vec![0, 1, 2, 50, 150, 243, 255]);
     dbg!(u8s2.intersection(&u8s).into_iter().collect::<Vec<u8>>() == vec![0]);
 }
