@@ -26,6 +26,7 @@ pub struct PyUtils {
     tensor_scale: PyObject,
     pub generalfunctions: std::collections::HashMap<String, PyObject>,
     pub gen_index_function: PyObject,
+    pub explicit_gen_index_function: PyObject,
     pub pow: PyObject,
     einsum: PyObject,
     make_diagonal: PyObject,
@@ -39,6 +40,7 @@ pub struct PyUtils {
     pub gc: Py<PyModule>,
     pub optimizing_symbolic_size_warning: PyObject,
     pub maybe_dtype_to_maybe_string: PyObject,
+    pub random_i64: PyObject,
 }
 
 /// misc python utilities
@@ -77,11 +79,8 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
             .unwrap()
             .extract()
             .unwrap(),
-        gen_index_function: utils
-            .getattr("gen_index_function")
-            .unwrap()
-            .extract()
-            .unwrap(),
+        gen_index_function: get("gen_index_function"),
+        explicit_gen_index_function: get("explicit_gen_index_function"),
         pow: get("pow"),
         einsum: get("einsum"),
         make_diagonal: get("make_diagonal"),
@@ -95,6 +94,7 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
         gc: PyModule::import(py, "gc").unwrap().into(),
         optimizing_symbolic_size_warning: get("OptimizingSymbolicSizeWarning"),
         maybe_dtype_to_maybe_string: get("maybe_dtype_to_maybe_string"),
+        random_i64: get("random_i64"),
     }
 });
 
@@ -154,6 +154,17 @@ pub fn scalar_to_tensor(v: f64, shape: Shape, device_dtype: TorchDeviceDtype) ->
         .context("scalar to tensor extract")
 }
 
+#[apply(make_py_func)]
+#[py_ident(py)]
+pub fn i64_to_tensor(v: i64, shape: Shape, device_dtype: TorchDeviceDtype) -> Result<Tensor> {
+    PY_UTILS
+        .scalar_to_tensor
+        .call(py, (v, shape, device_dtype), None)
+        .context("scalar to tensor")?
+        .extract::<Tensor>(py)
+        .context("scalar to tensor extract")
+}
+
 #[apply(make_py_func)]
 #[py_ident(py)]
 pub fn einsum(items: Vec<(Tensor, EinsumAxes)>, out_axes: EinsumAxes) -> PyResult<Tensor> {
@@ -207,6 +218,12 @@ pub fn tensor_scale(tensor: &Tensor) -> PyResult<f64> {
     PY_UTILS.tensor_scale.call(py, (tensor,), None)?.extract(py)
 }
 
+#[apply(make_py_func)]
+#[py_ident(py)]
+pub fn random_torch_i64() -> i64 {
+    PY_UTILS.random_i64.call0(py).unwrap().extract(py).unwrap()
+}
+
 pub struct PyEinops {
     pub einops: Py<PyModule>,
     pub repeat: PyObject,
