@@ -138,7 +138,8 @@ macro_rules! setup_wrap {
 
 #[macro_export]
 macro_rules! setup_callable {
-    ($struct_name:ident, $enum_name:ty, $from_py:ty, $func_name:ident ($($arg_name:ident : $arg_ty:ty),*) -> $func_ret:ty) => {
+    ($struct_name:ident, $enum_name:ty, $from_py:ty, $func_name:ident ($($arg_name:ident : $arg_ty:ty),*) -> $func_ret:ty,no_py_callable) => {
+
         $crate::setup_wrap!($struct_name, $enum_name, $from_py);
 
         paste::paste!{
@@ -152,13 +153,6 @@ macro_rules! setup_callable {
             }
         }
 
-        #[pymethods]
-        impl $struct_name {
-            fn __call__(&self, _py: Python<'_>, $($arg_name : $arg_ty,)*) -> anyhow::Result<$func_ret> {
-                self.$func_name($($arg_name,)*)
-            }
-        }
-
         impl $struct_name {
             pub fn new_func<F: Fn($($arg_ty,)*) -> $func_ret + Send + Sync + 'static>(func: F) -> Self {
                 Self::new_func_err(move |n| Ok(func(n)))
@@ -176,6 +170,17 @@ macro_rules! setup_callable {
                 }
             }
         }
+    };
+    ($struct_name:ident, $enum_name:ty, $from_py:ty, $func_name:ident ($($arg_name:ident : $arg_ty:ty),*) -> $func_ret:ty) => {
+
+        setup_callable!($struct_name, $enum_name, $from_py, $func_name ($($arg_name : $arg_ty),*) -> $func_ret,no_py_callable);
+
+        #[pymethods]
+        impl $struct_name {
+            fn __call__(&self, _py: Python<'_>, $($arg_name : $arg_ty,)*) -> anyhow::Result<$func_ret> {
+                self.$func_name($($arg_name,)*)
+            }
+        }
 
     };
 }
