@@ -126,6 +126,11 @@ tensor_hash_cache: Dict[int, Tuple[weakref.ref[torch.Tensor], bytes]] = {}
 
 
 def hash_tensor(tensor: torch.Tensor) -> bytes:
+    if isinstance(tensor, torch.nn.Parameter):
+        raise TypeError(
+            f"Circuits assumes tensors are immutable, but this tensor " + "is a Parameter! You should call .clone()."
+        )
+
     if (
         (cached := tensor_hash_cache.get(id(tensor))) is not None
         and (cached_tensor := cached[0]()) is not None
