@@ -26,9 +26,9 @@ x.args
 
 # %%
 
-assert isinstance(x.all_input_circuits()[0], Scalar)
-assert isinstance(x.all_input_circuits()[1], Symbol)
-assert isinstance(x.all_input_circuits()[2], Array)
+assert isinstance(x.children[0], Scalar)
+assert isinstance(x.children[1], Symbol)
+assert isinstance(x.children[2], Array)
 
 # %%
 
@@ -68,7 +68,7 @@ y = Add(
     Symbol((3, 1, 5), uuid.uuid4()),
     Array(torch.randn(3, 4, 1)),
 )
-print(y.nodes)
+print(y.children)
 print(y)
 
 # %%
@@ -119,12 +119,12 @@ print(i.index)
 # %%
 
 v1 = Concat(Array(torch.randn(2, 3)), Array(torch.randn(3, 3)), axis=0)
-v2 = Concat(*v1.nodes, axis=0)
+v2 = Concat(*v1.children, axis=0)
 assert v1.shape == (5, 3)
 assert v1 == v2
 
 v1 = Concat(Array(torch.randn(3, 3)), Array(torch.randn(3, 3)), axis=0)
-v2 = Concat(*v1.nodes, axis=1)
+v2 = Concat(*v1.children, axis=1)
 assert v1 != v2
 
 
