@@ -57,7 +57,7 @@ pub fn main() {
     opsettings.log_simplifications = true;
     opsettings.keep_all_names = true;
     for _ in 0..3 {
-        let mut ctx = OptimizationContext::new_settings(opsettings);
+        let mut ctx = OptimizationContext::new_settings(opsettings.clone());
         for circ in &circuits_allnamed {
             println!("{:?}", circ.info().name);
             timed!(optimize_circuit(circ.clone(), &mut ctx).unwrap());
