@@ -58,7 +58,9 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
             add_pull_scatter, einsum_pull_scatter, index_einsum_to_scatter, scatter_elim_identity,
             scatter_pull_removable_axes, scatter_to_concat,
         },
-        scheduled_execution::scheduled_evaluate,
+        scheduled_execution::{
+            py_circuit_to_schedule, py_circuit_to_schedule_many, scheduled_evaluate,
+        },
     };
     use pyo3::{exceptions::PyValueError, types::PyTuple, PyTypeInfo};
 
@@ -302,6 +304,8 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(scheduled_evaluate, m)?)?;
     m.add_function(wrap_pyfunction!(optimize_and_evaluate, m)?)?;
     m.add_function(wrap_pyfunction!(optimize_and_evaluate_many, m)?)?;
+    m.add_function(wrap_pyfunction!(py_circuit_to_schedule, m)?)?;
+    m.add_function(wrap_pyfunction!(py_circuit_to_schedule_many, m)?)?;
     m.add_function(wrap_pyfunction!(
         circuit_rewrites::circuit_optimizer::optimize_to_schedule,
         m
